@@ -13,7 +13,7 @@
  */
 const kDataCell = 1; // Indicates the cell is origin data cell
 const kRowHeaderCell = 2; // Indicates the cell is row header cell
-const kColHeaderCell = 4; // Indicated teh cell is column header cell
+const kColHeaderCell = 4; // Indicated the cell is column header cell
 const kOrigin = kDataCell | kRowHeaderCell | kColHeaderCell;
 
 const kRowSpanned = 8; // Indicates the cell is not origin and row spanned 
@@ -224,7 +224,7 @@ function testTableIndexes(aIdentifier, aIdxes)
 
       if (idx != - 1) {
 
-        // getRowAtIndex
+        // getRowIndexAt
         var origRowIdx = rowIdx;
         while (origRowIdx > 0 &&
                aIdxes[rowIdx][colIdx] == aIdxes[origRowIdx - 1][colIdx])
@@ -237,9 +237,9 @@ function testTableIndexes(aIdentifier, aIdxes)
         }
 
         is(obtainedRowIdx, origRowIdx,
-           id + ": row for index " + idx +" is not correct");
+           id + ": row for index " + idx + " is not correct (getRowIndexAt)");
 
-        // getColumnAtIndex
+        // getColumnIndexAt
         var origColIdx = colIdx;
         while (origColIdx > 0 &&
                aIdxes[rowIdx][colIdx] == aIdxes[rowIdx][origColIdx - 1])
@@ -252,7 +252,21 @@ function testTableIndexes(aIdentifier, aIdxes)
         }
 
         is(obtainedColIdx, origColIdx,
-           id + ": column  for index " + idx +" is not correct");
+           id + ": column  for index " + idx + " is not correct (getColumnIndexAt)");
+
+        // getRowAndColumnIndicesAt
+        var obtainedRowIdxObj = { }, obtainedColIdxObj = { };
+        try {
+          tableAcc.getRowAndColumnIndicesAt(idx, obtainedRowIdxObj,
+                                            obtainedColIdxObj);
+        } catch (e) {
+          ok(false, id + ": can't get row and column indices for cell index " + idx + "," + e);
+        }
+
+        is(obtainedRowIdxObj.value, origRowIdx,
+           id + ": row for index " + idx + " is not correct (getRowAndColumnIndicesAt)");
+        is(obtainedColIdxObj.value, origColIdx,
+           id + ": column  for index " + idx + " is not correct (getRowAndColumnIndicesAt)");
 
         if (cellAcc) {
 
