@@ -53,6 +53,8 @@
 // done rather than a byte range request.
 #define SEEK_VS_READ_THRESHOLD (32*1024)
 
+#define HTTP_REQUESTED_RANGE_NOT_SATISFIABLE_CODE 416
+
 class nsMediaDecoder;
 
 /**
@@ -132,11 +134,9 @@ private:
    this class must be created on the main thread. 
 
    Most methods must be called on the main thread only. Read, Seek and
-   Tell may be called on another thread which may be a non main
-   thread. They may not be called on multiple other threads though. In
-   the case of the Ogg Decoder they are called on the Decode thread
-   for example. You must ensure that no threads are calling these
-   methods once Close is called.
+   Tell must only be called on non-main threads. In the case of the Ogg
+   Decoder they are called on the Decode thread for example. You must
+   ensure that no threads are calling these methods once Close is called.
 
    Instances of this class are explicitly managed. 'delete' it when done.
 */
@@ -251,6 +251,16 @@ public:
   // nsMediaDecoder::NotifySuspendedStatusChanged is called when this
   // changes.
   virtual PRBool IsSuspendedByCache() = 0;
+  // Returns true if this stream has been suspended.
+  virtual PRBool IsSuspended() = 0;
+  // Reads only data which is cached in the media cache. If you try to read
+  // any data which overlaps uncached data, or if aCount bytes otherwise can't
+  // be read, this function will return failure. This function be called from
+  // any thread, and it is the only read operation which is safe to call on
+  // the main thread, since it's guaranteed to be non blocking.
+  virtual nsresult ReadFromCache(char* aBuffer,
+                                 PRInt64 aOffset,
+                                 PRUint32 aCount) = 0;
 
   /**
    * Create a stream, reading data from the media resource via the
@@ -275,6 +285,11 @@ protected:
     MOZ_COUNT_CTOR(nsMediaStream);
   }
 
+  // Set the request's load flags to aFlags.  If the request is part of a
+  // load group, the request is removed from the group, the flags are set, and
+  // then the request is added back to the load group.
+  void ModifyLoadFlags(nsLoadFlags aFlags);
+
   // This is not an nsCOMPointer to prevent a circular reference
   // between the decoder to the media stream object. The stream never
   // outlives the lifetime of the decoder.
@@ -340,6 +355,7 @@ public:
   // Return PR_TRUE if the stream has been closed.
   PRBool IsClosed() const { return mCacheStream.IsClosed(); }
   virtual nsMediaStream* CloneData(nsMediaDecoder* aDecoder);
+  virtual nsresult ReadFromCache(char* aBuffer, PRInt64 aOffset, PRUint32 aCount);
 
   // Other thread
   virtual void     SetReadMode(nsMediaCacheStream::ReadMode aMode);
@@ -357,6 +373,7 @@ public:
   virtual PRInt64 GetCachedDataEnd(PRInt64 aOffset);
   virtual PRBool  IsDataCachedToEndOfStream(PRInt64 aOffset);
   virtual PRBool  IsSuspendedByCache();
+  virtual PRBool  IsSuspended();
 
   class Listener : public nsIStreamListener,
                    public nsIInterfaceRequestor,
@@ -391,6 +408,7 @@ protected:
   // if possible. Main thread only.
   nsresult OpenChannel(nsIStreamListener** aStreamListener);
   nsresult RecreateChannel();
+  // Add headers to HTTP request. Main thread only.
   void SetupChannelHeaders();
   // Closes the channel. Main thread only.
   void CloseChannel();
@@ -409,7 +427,7 @@ protected:
   nsRefPtr<Listener> mListener;
   // A data received event for the decoder that has been dispatched but has
   // not yet been processed.
-  nsRevocableEventPtr<nsNonOwningRunnableMethod<nsMediaChannelStream> > mDataReceivedEvent;
+  nsRevocableEventPtr<nsRunnableMethod<nsMediaChannelStream, void, false> > mDataReceivedEvent;
   PRUint32           mSuspendCount;
   // When this flag is set, if we get a network error we should silently
   // reopen the stream.
