@@ -68,8 +68,17 @@ class nsSVGBoolean;
 class nsSVGEnum;
 struct nsSVGEnumMapping;
 class nsSVGViewBox;
-class nsSVGPreserveAspectRatio;
 class nsSVGString;
+struct gfxMatrix;
+namespace mozilla {
+class SVGAnimatedNumberList;
+class SVGNumberList;
+class SVGAnimatedLengthList;
+class SVGUserUnitList;
+class SVGAnimatedPointList;
+class SVGAnimatedPathSegList;
+class SVGAnimatedPreserveAspectRatio;
+}
 
 typedef nsStyledElement nsSVGElementBase;
 
@@ -77,11 +86,19 @@ class nsSVGElement : public nsSVGElementBase,    // nsIContent
                      public nsISVGValueObserver  // :nsISupportsWeakReference
 {
 protected:
-  nsSVGElement(nsINodeInfo *aNodeInfo);
+  nsSVGElement(already_AddRefed<nsINodeInfo> aNodeInfo);
   nsresult Init();
   virtual ~nsSVGElement();
 
 public:
+  typedef mozilla::SVGNumberList SVGNumberList;
+  typedef mozilla::SVGAnimatedNumberList SVGAnimatedNumberList;
+  typedef mozilla::SVGUserUnitList SVGUserUnitList;
+  typedef mozilla::SVGAnimatedLengthList SVGAnimatedLengthList;
+  typedef mozilla::SVGAnimatedPointList SVGAnimatedPointList;
+  typedef mozilla::SVGAnimatedPathSegList SVGAnimatedPathSegList;
+  typedef mozilla::SVGAnimatedPreserveAspectRatio SVGAnimatedPreserveAspectRatio;
+
   // nsISupports
   NS_DECL_ISUPPORTS_INHERITED
 
@@ -99,8 +116,6 @@ public:
 
   virtual PRBool IsNodeOfType(PRUint32 aFlags) const;
 
-  virtual already_AddRefed<nsIURI> GetBaseURI() const;
-
   NS_IMETHOD WalkContentStyleRules(nsRuleWalker* aRuleWalker);
 
   static const MappedAttributeEntry sFillStrokeMap[];
@@ -145,6 +160,17 @@ public:
    */
   virtual gfxMatrix PrependLocalTransformTo(const gfxMatrix &aMatrix);
 
+  // Setter for to set the current <animateMotion> transformation
+  // Only visible for nsSVGGraphicElement, so it's a no-op here, and that
+  // subclass has the useful implementation.
+  virtual void SetAnimateMotionTransform(const gfxMatrix* aMatrix) {/*no-op*/}
+
+  PRBool IsStringAnimatable(PRUint8 aAttrEnum) {
+    return GetStringInfo().mStringInfo[aAttrEnum].mIsAnimatable;
+  }
+  PRBool NumberAttrAllowsPercentage(PRUint8 aAttrEnum) {
+    return GetNumberInfo().mNumberInfo[aAttrEnum].mPercentagesAllowed;
+  }
   virtual void DidChangeLength(PRUint8 aAttrEnum, PRBool aDoSetAttr);
   virtual void DidChangeNumber(PRUint8 aAttrEnum, PRBool aDoSetAttr);
   virtual void DidChangeInteger(PRUint8 aAttrEnum, PRBool aDoSetAttr);
@@ -153,18 +179,52 @@ public:
   virtual void DidChangeEnum(PRUint8 aAttrEnum, PRBool aDoSetAttr);
   virtual void DidChangeViewBox(PRBool aDoSetAttr);
   virtual void DidChangePreserveAspectRatio(PRBool aDoSetAttr);
+  virtual void DidChangeNumberList(PRUint8 aAttrEnum, PRBool aDoSetAttr);
+  virtual void DidChangeLengthList(PRUint8 aAttrEnum, PRBool aDoSetAttr);
+  virtual void DidChangePointList(PRBool aDoSetAttr);
+  virtual void DidChangePathSegList(PRBool aDoSetAttr);
   virtual void DidChangeString(PRUint8 aAttrEnum) {}
 
-  void DidAnimateLength(PRUint8 aAttrEnum);
+  virtual void DidAnimateLength(PRUint8 aAttrEnum);
+  virtual void DidAnimateNumber(PRUint8 aAttrEnum);
+  virtual void DidAnimateInteger(PRUint8 aAttrEnum);
+  virtual void DidAnimateAngle(PRUint8 aAttrEnum);
+  virtual void DidAnimateBoolean(PRUint8 aAttrEnum);
+  virtual void DidAnimateEnum(PRUint8 aAttrEnum);
+  virtual void DidAnimateViewBox();
+  virtual void DidAnimatePreserveAspectRatio();
+  virtual void DidAnimateNumberList(PRUint8 aAttrEnum);
+  virtual void DidAnimateLengthList(PRUint8 aAttrEnum);
+  virtual void DidAnimatePointList();
+  virtual void DidAnimatePathSegList();
+  virtual void DidAnimateTransform();
+  virtual void DidAnimateString(PRUint8 aAttrEnum);
 
   void GetAnimatedLengthValues(float *aFirst, ...);
   void GetAnimatedNumberValues(float *aFirst, ...);
   void GetAnimatedIntegerValues(PRInt32 *aFirst, ...);
+  SVGAnimatedNumberList* GetAnimatedNumberList(PRUint8 aAttrEnum);
+  SVGAnimatedNumberList* GetAnimatedNumberList(nsIAtom *aAttrName);
+  void GetAnimatedLengthListValues(SVGUserUnitList *aFirst, ...);
+  SVGAnimatedLengthList* GetAnimatedLengthList(PRUint8 aAttrEnum);
+  virtual SVGAnimatedPointList* GetAnimatedPointList() {
+    return nsnull;
+  }
+  virtual SVGAnimatedPathSegList* GetAnimPathSegList() {
+    // DOM interface 'SVGAnimatedPathData' (*inherited* by nsSVGPathElement)
+    // has a member called 'animatedPathSegList' member, so we have a shorter
+    // name so we don't get hidden by the GetAnimatedPathSegList declared by
+    // NS_DECL_NSIDOMSVGANIMATEDPATHDATA.
+    return nsnull;
+  }
 
 #ifdef MOZ_SMIL
-  virtual nsISMILAttr* GetAnimatedAttr(const nsIAtom* aName);
+  virtual nsISMILAttr* GetAnimatedAttr(PRInt32 aNamespaceID, nsIAtom* aName);
   void AnimationNeedsResample();
   void FlushAnimations();
+#else
+  void AnimationNeedsResample() { /* do nothing */ }
+  void FlushAnimations() { /* do nothing */ }
 #endif
 
   virtual void RecompileScriptEventListeners();
@@ -172,6 +232,13 @@ public:
   void GetStringBaseValue(PRUint8 aAttrEnum, nsAString& aResult) const;
   void SetStringBaseValue(PRUint8 aAttrEnum, const nsAString& aValue);
 
+  virtual nsIAtom* GetPointListAttrName() const {
+    return nsnull;
+  }
+  virtual nsIAtom* GetPathDataAttrName() const {
+    return nsnull;
+  }
+
 protected:
   virtual nsresult AfterSetAttr(PRInt32 aNamespaceID, nsIAtom* aName,
                                 const nsAString* aValue, PRBool aNotify);
@@ -185,6 +252,11 @@ protected:
   virtual PRBool IsEventName(nsIAtom* aName);
 
   void UpdateContentStyleRule();
+#ifdef MOZ_SMIL
+  void UpdateAnimatedContentStyleRule();
+  nsICSSStyleRule* GetAnimatedContentStyleRule();
+#endif // MOZ_SMIL
+
   nsISVGValue* GetMappedAttribute(PRInt32 aNamespaceID, nsIAtom* aName);
   nsresult AddMappedSVGValue(nsIAtom* aName, nsISupports* aValue,
                              PRInt32 aNamespaceID = kNameSpaceID_None);
@@ -215,6 +287,7 @@ protected:
   struct NumberInfo {
     nsIAtom** mName;
     float     mDefaultValue;
+    PRPackedBool mPercentagesAllowed;
   };
 
   struct NumberAttributesInfo {
@@ -311,9 +384,60 @@ protected:
     void Reset(PRUint8 aAttrEnum);
   };
 
+  struct NumberListInfo {
+    nsIAtom** mName;
+  };
+
+  struct NumberListAttributesInfo {
+    SVGAnimatedNumberList* mNumberLists;
+    NumberListInfo*        mNumberListInfo;
+    PRUint32               mNumberListCount;
+
+    NumberListAttributesInfo(SVGAnimatedNumberList *aNumberLists,
+                             NumberListInfo *aNumberListInfo,
+                             PRUint32 aNumberListCount)
+      : mNumberLists(aNumberLists)
+      , mNumberListInfo(aNumberListInfo)
+      , mNumberListCount(aNumberListCount)
+    {}
+
+    void Reset(PRUint8 aAttrEnum);
+  };
+
+  struct LengthListInfo {
+    nsIAtom** mName;
+    PRUint8   mAxis;
+    /**
+     * Flag to indicate whether appending zeros to the end of the list would
+     * change the rendering of the SVG for the attribute in question. For x and
+     * y on the <text> element this is true, but for dx and dy on <text> this
+     * is false. This flag is fed down to SVGLengthListSMILType so it can
+     * determine if it can sensibly animate from-to lists of different lengths,
+     * which is desirable in the case of dx and dy.
+     */
+    PRPackedBool mCouldZeroPadList;
+  };
+
+  struct LengthListAttributesInfo {
+    SVGAnimatedLengthList* mLengthLists;
+    LengthListInfo*        mLengthListInfo;
+    PRUint32               mLengthListCount;
+
+    LengthListAttributesInfo(SVGAnimatedLengthList *aLengthLists,
+                             LengthListInfo *aLengthListInfo,
+                             PRUint32 aLengthListCount)
+      : mLengthLists(aLengthLists)
+      , mLengthListInfo(aLengthListInfo)
+      , mLengthListCount(aLengthListCount)
+    {}
+
+    void Reset(PRUint8 aAttrEnum);
+  };
+
   struct StringInfo {
     nsIAtom**    mName;
     PRInt32      mNamespaceID;
+    PRPackedBool mIsAnimatable;
   };
 
   struct StringAttributesInfo {
@@ -339,7 +463,9 @@ protected:
   // We assume all viewboxes and preserveAspectRatios are alike
   // so we don't need to wrap the class
   virtual nsSVGViewBox *GetViewBox();
-  virtual nsSVGPreserveAspectRatio *GetPreserveAspectRatio();
+  virtual SVGAnimatedPreserveAspectRatio *GetPreserveAspectRatio();
+  virtual NumberListAttributesInfo GetNumberListInfo();
+  virtual LengthListAttributesInfo GetLengthListInfo();
   virtual StringAttributesInfo GetStringInfo();
 
   static nsSVGEnumMapping sSVGUnitTypesMap[];
@@ -357,6 +483,20 @@ private:
 
   void ResetOldStyleBaseType(nsISVGValue *svg_value);
 
+  struct ObservableModificationData {
+    // Only to be used if |name| is non-null.  Otherwise, modType will
+    // be 0 to indicate NS_OK should be returned and 1 to indicate
+    // NS_ERROR_UNEXPECTED should be returned.
+    ObservableModificationData(const nsAttrName* aName, PRUint32 aModType):
+      name(aName), modType(aModType)
+    {}
+    const nsAttrName* name;
+    PRUint8 modType;
+  };
+  ObservableModificationData
+    GetModificationDataForObservable(nsISVGValue* aObservable,
+                                     nsISVGValue::modificationType aModType);
+
   nsCOMPtr<nsICSSStyleRule> mContentStyleRule;
   nsAttrAndChildArray mMappedAttributes;
 
@@ -369,28 +509,47 @@ private:
 #define NS_IMPL_NS_NEW_SVG_ELEMENT(_elementName)                             \
 nsresult                                                                     \
 NS_NewSVG##_elementName##Element(nsIContent **aResult,                       \
-                                 nsINodeInfo *aNodeInfo)                     \
+                                 already_AddRefed<nsINodeInfo> aNodeInfo)    \
 {                                                                            \
-  nsSVG##_elementName##Element *it =                                         \
+  nsRefPtr<nsSVG##_elementName##Element> it =                                \
     new nsSVG##_elementName##Element(aNodeInfo);                             \
   if (!it)                                                                   \
     return NS_ERROR_OUT_OF_MEMORY;                                           \
                                                                              \
-  NS_ADDREF(it);                                                             \
-                                                                             \
   nsresult rv = it->Init();                                                  \
                                                                              \
   if (NS_FAILED(rv)) {                                                       \
-    NS_RELEASE(it);                                                          \
     return rv;                                                               \
   }                                                                          \
                                                                              \
-  *aResult = it;                                                             \
+  *aResult = it.forget().get();                                              \
                                                                              \
   return rv;                                                                 \
 }
 
-// No unlinking, we'd need to null out the value pointer (the object it
+#define NS_IMPL_NS_NEW_SVG_ELEMENT_CHECK_PARSER(_elementName)                \
+nsresult                                                                     \
+NS_NewSVG##_elementName##Element(nsIContent **aResult,                       \
+                                 already_AddRefed<nsINodeInfo> aNodeInfo,    \
+                                 FromParser aFromParser)                     \
+{                                                                            \
+  nsRefPtr<nsSVG##_elementName##Element> it =                                \
+    new nsSVG##_elementName##Element(aNodeInfo, aFromParser);                \
+  if (!it)                                                                   \
+    return NS_ERROR_OUT_OF_MEMORY;                                           \
+                                                                             \
+  nsresult rv = it->Init();                                                  \
+                                                                             \
+  if (NS_FAILED(rv)) {                                                       \
+    return rv;                                                               \
+  }                                                                          \
+                                                                             \
+  *aResult = it.forget().get();                                              \
+                                                                             \
+  return rv;                                                                 \
+}
+
+ // No unlinking, we'd need to null out the value pointer (the object it
 // points to is held by the element) and null-check it everywhere.
 #define NS_SVG_VAL_IMPL_CYCLE_COLLECTION(_val, _element)                     \
 NS_IMPL_CYCLE_COLLECTION_CLASS(_val)                                         \
