@@ -49,13 +49,13 @@
 #include "nsXBLService.h"
 #include "nsIServiceManager.h"
 #include "nsGkAtoms.h"
-#include "nsIXBLDocumentInfo.h"
+#include "nsXBLDocumentInfo.h"
 #include "nsIDOMElement.h"
 #include "nsINativeKeyBindings.h"
 #include "nsIController.h"
 #include "nsIControllers.h"
 #include "nsIDOMWindowInternal.h"
-#include "nsIFocusController.h"
+#include "nsFocusManager.h"
 #include "nsPIWindowRoot.h"
 #include "nsIURI.h"
 #include "nsNetUtil.h"
@@ -65,7 +65,6 @@
 #include "nsIDOMNSDocument.h"
 #include "nsPIWindowRoot.h"
 #include "nsPIDOMWindow.h"
-#include "nsIFocusController.h"
 #include "nsIDocShell.h"
 #include "nsIPresShell.h"
 #include "nsIPrivateDOMEvent.h"
@@ -77,8 +76,8 @@ static nsINativeKeyBindings *sNativeEditorBindings = nsnull;
 class nsXBLSpecialDocInfo
 {
 public:
-  nsCOMPtr<nsIXBLDocumentInfo> mHTMLBindings;
-  nsCOMPtr<nsIXBLDocumentInfo> mUserHTMLBindings;
+  nsRefPtr<nsXBLDocumentInfo> mHTMLBindings;
+  nsRefPtr<nsXBLDocumentInfo> mUserHTMLBindings;
 
   static const char sHTMLBindingStr[];
   static const char sUserHTMLBindingStr[];
@@ -90,7 +89,7 @@ public:
   void GetAllHandlers(const char* aType,
                       nsXBLPrototypeHandler** handler,
                       nsXBLPrototypeHandler** userHandler);
-  void GetHandlers(nsIXBLDocumentInfo* aInfo,
+  void GetHandlers(nsXBLDocumentInfo* aInfo,
                    const nsACString& aRef,
                    nsXBLPrototypeHandler** aResult);
 
@@ -145,12 +144,11 @@ void nsXBLSpecialDocInfo::LoadDocInfo()
 //
 // 
 void
-nsXBLSpecialDocInfo::GetHandlers(nsIXBLDocumentInfo* aInfo,
+nsXBLSpecialDocInfo::GetHandlers(nsXBLDocumentInfo* aInfo,
                                  const nsACString& aRef,
                                  nsXBLPrototypeHandler** aResult)
 {
-  nsXBLPrototypeBinding* binding;
-  aInfo->GetPrototypeBinding(aRef, &binding);
+  nsXBLPrototypeBinding* binding = aInfo->GetPrototypeBinding(aRef);
   
   NS_ASSERTION(binding, "No binding found for the XBL window key handler.");
   if (!binding)
@@ -360,18 +358,13 @@ nsXBLWindowKeyHandler::WalkHandlers(nsIDOMKeyEvent* aKeyEvent, nsIAtom* aEventTy
 
   WalkHandlersInternal(aKeyEvent, aEventType, mHandler);
 
-  nsINativeKeyBindings *nativeBindings;
-  if (isEditor && (nativeBindings = GetEditorKeyBindings())) {
+  if (isEditor && GetEditorKeyBindings()) {
     nsNativeKeyEvent nativeEvent;
     // get the DOM window we're attached to
     nsCOMPtr<nsIControllers> controllers;
     nsCOMPtr<nsPIWindowRoot> root = do_QueryInterface(mTarget);
     if (root) {
-      nsCOMPtr<nsIFocusController> fc;
-      root->GetFocusController(getter_AddRefs(fc));
-      if (fc) {
-        fc->GetControllers(getter_AddRefs(controllers));
-      }
+      root->GetControllers(getter_AddRefs(controllers));
     }
 
     PRBool handled = PR_FALSE;
@@ -449,20 +442,18 @@ nsXBLWindowKeyHandler::ShutDown()
 PRBool
 nsXBLWindowKeyHandler::IsEditor()
 {
-  nsCOMPtr<nsPIWindowRoot> windowRoot(do_QueryInterface(mTarget));
-  NS_ENSURE_TRUE(windowRoot, PR_FALSE);
-  nsCOMPtr<nsIFocusController> focusController;
-  windowRoot->GetFocusController(getter_AddRefs(focusController));
-  if (!focusController) {
-    NS_WARNING("********* Something went wrong! No focus controller on the root!!!\n");
+  // XXXndeakin even though this is only used for key events which should be
+  // going to the focused frame anyway, this doesn't seem like the right way
+  // to determine if something is an editor.
+  nsIFocusManager* fm = nsFocusManager::GetFocusManager();
+  if (!fm)
     return PR_FALSE;
-  }
 
-  nsCOMPtr<nsIDOMWindowInternal> focusedWindow;
-  focusController->GetFocusedWindow(getter_AddRefs(focusedWindow));
+  nsCOMPtr<nsIDOMWindow> focusedWindow;
+  fm->GetFocusedWindow(getter_AddRefs(focusedWindow));
   if (!focusedWindow)
     return PR_FALSE;
-  
+
   nsCOMPtr<nsPIDOMWindow> piwin(do_QueryInterface(focusedWindow));
   nsIDocShell *docShell = piwin->GetDocShell();
   nsCOMPtr<nsIPresShell> presShell;
@@ -470,9 +461,7 @@ nsXBLWindowKeyHandler::IsEditor()
     docShell->GetPresShell(getter_AddRefs(presShell));
 
   if (presShell) {
-    PRInt16 isEditor;
-    presShell->GetSelectionFlags(&isEditor);
-    return isEditor == nsISelectionDisplay::DISPLAY_ALL;
+    return presShell->GetSelectionFlags() == nsISelectionDisplay::DISPLAY_ALL;
   }
 
   return PR_FALSE;
@@ -546,13 +535,12 @@ nsXBLWindowKeyHandler::WalkHandlersAndExecute(nsIDOMKeyEvent* aKeyEvent,
         // Locate the command element in question.  Note that we
         // know "elt" is in a doc if we're dealing with it here.
         NS_ASSERTION(elt->IsInDoc(), "elt must be in document");
-        nsCOMPtr<nsIDOMDocument> domDoc(
-           do_QueryInterface(elt->GetCurrentDoc()));
-        if (domDoc)
-          domDoc->GetElementById(command, getter_AddRefs(commandElt));
+        nsIDocument *doc = elt->GetCurrentDoc();
+        if (doc)
+          commandElt = do_QueryInterface(doc->GetElementById(command));
 
         if (!commandElt) {
-          NS_ERROR("A XUL <key> is observing a command that doesn't exist. Unable to execute key binding!\n");
+          NS_ERROR("A XUL <key> is observing a command that doesn't exist. Unable to execute key binding!");
           continue;
         }
       }
