@@ -54,6 +54,7 @@
 #include "txLog.h"
 #include "nsUnicharUtils.h"
 #include "nsAttrName.h"
+#include "nsTArray.h"
 
 const PRUint32 kUnknownIndex = PRUint32(-1);
 
@@ -106,13 +107,7 @@ txXPathTreeWalker::moveToElementById(const nsAString& aID)
 
     nsCOMPtr<nsIContent> content;
     if (doc) {
-        nsCOMPtr<nsIDOMDocument> document = do_QueryInterface(doc);
-        NS_ASSERTION(document, "QI failed");
-
-        nsCOMPtr<nsIDOMElement> element;
-        document->GetElementById(aID, getter_AddRefs(element));
-
-        content = do_QueryInterface(element);
+        content = doc->GetElementById(aID);
     }
     else {
         // We're in a disconnected subtree, search only that subtree.
@@ -209,9 +204,9 @@ txXPathTreeWalker::moveToFirstChild()
     }
 
     NS_ASSERTION(!mPosition.isDocument() ||
-                 (mCurrentIndex == kUnknownIndex && !mDescendants.Count()),
+                 (mCurrentIndex == kUnknownIndex && mDescendants.IsEmpty()),
                  "we shouldn't have any position info at the document");
-    NS_ASSERTION(mCurrentIndex != kUnknownIndex || !mDescendants.Count(),
+    NS_ASSERTION(mCurrentIndex != kUnknownIndex || mDescendants.IsEmpty(),
                  "Index should be known if parents index are");
 
     nsIContent* child = mPosition.mNode->GetChildAt(0);
@@ -238,9 +233,9 @@ txXPathTreeWalker::moveToLastChild()
     }
 
     NS_ASSERTION(!mPosition.isDocument() ||
-                 (mCurrentIndex == kUnknownIndex && !mDescendants.Count()),
+                 (mCurrentIndex == kUnknownIndex && mDescendants.IsEmpty()),
                  "we shouldn't have any position info at the document");
-    NS_ASSERTION(mCurrentIndex != kUnknownIndex || !mDescendants.Count(),
+    NS_ASSERTION(mCurrentIndex != kUnknownIndex || mDescendants.IsEmpty(),
                  "Index should be known if parents index are");
 
     PRUint32 total = mPosition.mNode->GetChildCount();
@@ -296,7 +291,7 @@ txXPathTreeWalker::moveToParent()
         return PR_FALSE;
     }
 
-    PRInt32 count = mDescendants.Count();
+    PRUint32 count = mDescendants.Length();
     if (count) {
         mCurrentIndex = mDescendants.ValueAt(--count);
         mDescendants.RemoveValueAt(count);
@@ -381,7 +376,7 @@ txXPathNodeUtils::getLocalName(const txXPathNode& aNode)
     }
 
     if (aNode.isContent()) {
-        if (aNode.mNode->IsNodeOfType(nsINode::eELEMENT)) {
+        if (aNode.mNode->IsElement()) {
             nsIAtom* localName = aNode.Content()->Tag();
             NS_ADDREF(localName);
 
@@ -433,16 +428,9 @@ txXPathNodeUtils::getLocalName(const txXPathNode& aNode, nsAString& aLocalName)
     }
 
     if (aNode.isContent()) {
-        if (aNode.mNode->IsNodeOfType(nsINode::eELEMENT)) {
+        if (aNode.mNode->IsElement()) {
             nsINodeInfo* nodeInfo = aNode.Content()->NodeInfo();
             nodeInfo->GetLocalName(aLocalName);
-
-            // Check for html
-            if (nodeInfo->NamespaceEquals(kNameSpaceID_None) &&
-                aNode.mNode->IsNodeOfType(nsINode::eHTML)) {
-                ToUpperCase(aLocalName);
-            }
-
             return;
         }
 
@@ -464,7 +452,7 @@ txXPathNodeUtils::getLocalName(const txXPathNode& aNode, nsAString& aLocalName)
 
     // Check for html
     if (aNode.Content()->NodeInfo()->NamespaceEquals(kNameSpaceID_None) &&
-        aNode.Content()->IsNodeOfType(nsINode::eHTML)) {
+        aNode.Content()->IsHTML()) {
         ToUpperCase(aLocalName);
     }
 }
@@ -480,16 +468,15 @@ txXPathNodeUtils::getNodeName(const txXPathNode& aNode, nsAString& aName)
     }
 
     if (aNode.isContent()) {
-        if (aNode.mNode->IsNodeOfType(nsINode::eELEMENT)) {
+        if (aNode.mNode->IsElement()) {
             nsINodeInfo* nodeInfo = aNode.Content()->NodeInfo();
             nodeInfo->GetQualifiedName(aName);
 
             // Check for html
-            if (nodeInfo->NamespaceEquals(kNameSpaceID_None) &&
-                aNode.Content()->IsNodeOfType(nsINode::eHTML)) {
+            if (aNode.Content()->IsHTML() &&
+                aNode.Content()->IsInHTMLDocument()) {
                 ToUpperCase(aName);
             }
-
             return;
         }
 
@@ -509,8 +496,7 @@ txXPathNodeUtils::getNodeName(const txXPathNode& aNode, nsAString& aName)
     aNode.Content()->GetAttrNameAt(aNode.mIndex)->GetQualifiedName(aName);
 
     // Check for html
-    if (aNode.Content()->NodeInfo()->NamespaceEquals(kNameSpaceID_None) &&
-        aNode.Content()->IsNodeOfType(nsINode::eHTML)) {
+    if (aNode.Content()->IsHTML()) {
         ToUpperCase(aName);
     }
 }
@@ -578,7 +564,7 @@ txXPathNodeUtils::appendNodeValue(const txXPathNode& aNode, nsAString& aResult)
     }
 
     if (aNode.isDocument() ||
-        aNode.mNode->IsNodeOfType(nsINode::eELEMENT) ||
+        aNode.mNode->IsElement() ||
         aNode.mNode->IsNodeOfType(nsINode::eDOCUMENT_FRAGMENT)) {
         nsContentUtils::AppendNodeTextContent(aNode.mNode, PR_TRUE, aResult);
 
@@ -628,15 +614,17 @@ const char gPrintfFmtAttr[] = "id0x%016p-%010i";
 /* static */
 nsresult
 txXPathNodeUtils::getXSLTId(const txXPathNode& aNode,
+                            const txXPathNode& aBase,
                             nsAString& aResult)
 {
+    PRUword nodeid = ((PRUword)aNode.mNode) - ((PRUword)aBase.mNode);
     if (!aNode.isAttribute()) {
-        CopyASCIItoUTF16(nsPrintfCString(kFmtSize, gPrintfFmt, aNode.mNode),
+        CopyASCIItoUTF16(nsPrintfCString(kFmtSize, gPrintfFmt, nodeid),
                          aResult);
     }
     else {
         CopyASCIItoUTF16(nsPrintfCString(kFmtSizeAttr, gPrintfFmtAttr,
-                                         aNode.mNode, aNode.mIndex), aResult);
+                                         nodeid, aNode.mIndex), aResult);
     }
 
     return NS_OK;
@@ -691,7 +679,7 @@ txXPathNodeUtils::comparePosition(const txXPathNode& aNode,
     // same tree.
 
     // Get parents up the tree.
-    nsAutoVoidArray parents, otherParents;
+    nsAutoTArray<nsINode*, 8> parents, otherParents;
     nsINode* node = aNode.mNode;
     nsINode* otherNode = aOtherNode.mNode;
     nsINode* parent, *otherParent;
@@ -727,17 +715,16 @@ txXPathNodeUtils::comparePosition(const txXPathNode& aNode,
     }
 
     // Walk back down along the parent-chains until we find where they split.
-    PRInt32 total = parents.Count() - 1;
-    PRInt32 otherTotal = otherParents.Count() - 1;
+    PRInt32 total = parents.Length() - 1;
+    PRInt32 otherTotal = otherParents.Length() - 1;
     NS_ASSERTION(total != otherTotal, "Can't have same number of parents");
 
     PRInt32 lastIndex = PR_MIN(total, otherTotal);
     PRInt32 i;
     parent = nsnull;
     for (i = 0; i <= lastIndex; ++i) {
-        node = static_cast<nsINode*>(parents.ElementAt(total - i));
-        otherNode = static_cast<nsINode*>
-                               (otherParents.ElementAt(otherTotal - i));
+        node = parents.ElementAt(total - i);
+        otherNode = otherParents.ElementAt(otherTotal - i);
         if (node != otherNode) {
             if (!parent) {
                 // The two nodes are in different orphan subtrees.
@@ -837,13 +824,13 @@ txXPathNativeNode::getNode(const txXPathNode& aNode, nsIDOMNode** aResult)
 
     const nsAttrName* name = aNode.Content()->GetAttrNameAt(aNode.mIndex);
 
-    nsAutoString namespaceURI, localname;
+    nsAutoString namespaceURI;
     nsContentUtils::NameSpaceManager()->GetNameSpaceURI(name->NamespaceID(), namespaceURI);
-    name->LocalName()->ToString(localname);
 
     nsCOMPtr<nsIDOMElement> element = do_QueryInterface(aNode.mNode);
     nsCOMPtr<nsIDOMAttr> attr;
-    element->GetAttributeNodeNS(namespaceURI, localname,
+    element->GetAttributeNodeNS(namespaceURI,
+                                nsDependentAtomString(name->LocalName()),
                                 getter_AddRefs(attr));
 
     return CallQueryInterface(attr, aResult);
