@@ -36,42 +36,28 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '7.4.2-6-n.js';
 
 /**
-   File Name:          7.4.2-6.js
-   ECMA Section:       7.4.2
-
-   Description:
-   The following tokens are ECMAScript keywords and may not be used as
-   identifiers in ECMAScript programs.
-
-   Syntax
-
-   Keyword :: one of
-   break          for         new         var
-   continue       function    return      void
-   delete         if          this        while
-   else           in          typeof      with
-
-   This test verifies that the keyword cannot be used as an identifier.
-   Functioinal tests of the keyword may be found in the section corresponding
-   to the function of the keyword.
+   File Name:          7.5-4-n.js
+   ECMA Section:       7.5 Identifiers
+   Description:        Identifiers are of unlimited length
+   - can contain letters, a decimal digit, _, or $
+   - the first character cannot be a decimal digit
+   - identifiers are case sensitive
 
    Author:             christine@netscape.com
-   Date:               12 november 1997
-
+   Date:               11 september 1997
 */
-var SECTION = "7.4.2-6-n";
+var SECTION = "7.5-4-n";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "Keywords";
+var TITLE   = "Identifiers";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-DESCRIPTION = "var function = true";
+DESCRIPTION = "var 2abc";
 EXPECTED = "error";
 
-new TestCase( SECTION,  "var function = true",     "error",    eval("var function = true") );
+new TestCase( SECTION,    "var 2abc",   "error",    eval("var 2abc") );
 
 test();
