@@ -19,7 +19,8 @@
  * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Georgi Guninski
+ * Contributor(s): Reto Laemmler
+ *                 Brendan Eich
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,45 +37,52 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 336409;
-var summary = 'Integer overflow in js_obj_toSource';
-var actual = 'No Crash';
-var expect = /(No Crash|InternalError: script stack space quota is exhausted|InternalError: allocation size overflow)/;
+var BUGNUMBER = 354541;
+var summary = 'Regression to standard class constructors in case labels';
+var actual = '';
+var expect = '';
+
 
 printBugNumber(BUGNUMBER);
-printStatus (summary);
+printStatus (summary + ': top level');
 
-expectExitCode(0);
-expectExitCode(5);
+String.prototype.trim = function() { print('hallo'); };
 
-function createString(n)
+String.prototype.trim = function() { return 'hallo'; };
+
+const S = String;
+const Sp = String.prototype;
+
+expect = 'hallo';
+var expectStringInvariant = true
+  var actualStringInvariant;
+var expectStringPrototypeInvariant = true;
+var actualStringPrototypeInvariant;
+
+if (typeof Script == 'undefined')
 {
-  var l = n*1024*1024;
-  var r = 'r';
-
-  while (r.length < l)
+  print('Test skipped. Script not defined.');
+  reportCompare("Script not defined, Test skipped.",
+                "Script not defined, Test skipped.",
+                summary);
+}
+else
+{
+  var s = Script('var tmp = function(o) { switch(o) { case String: case 1: return ""; } }; actualStringInvariant = (String === S); actualStringPrototypeInvariant = (String.prototype === Sp); actual = "".trim();');
+  try
   {
-    r = r + r;
+    s();
   }
-  return r;
-}
+  catch(ex)
+  {
+    actual = ex + '';
+  }
+  reportCompare(expect, actual, 'trim() returned');
+  reportCompare(expectStringInvariant, actualStringInvariant,
+                'String invariant');
+  reportCompare(expectStringPrototypeInvariant,
+                actualStringPrototypeInvariant,
+                'String.prototype invariant');
 
-try
-{
-  var n = 128;
-  printStatus('Creating ' + n + 'MB string');
-  var r = createString(n);
-  printStatus('Done. length = ' + r.length);
-  printStatus('Creating object');
-  var o = {f1: r, f2: r, f3: r,f4: r,f5: r, f6: r, f7: r, f8: r,f9: r};
-  printStatus('object.toSource()');
-  var rr = o.toSource();
-  printStatus('Done.');
 }
-catch(ex)
-{
-  actual = ex + '';
-  print(actual);
-}
-
-reportMatch(expect, actual, summary);
+ 
