@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Gary Kwong
+ * Contributor(s): Igor Bukanov
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,8 +36,8 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 470388;
-var summary = 'TM: Do not assert: !(fp->flags & JSFRAME_POP_BLOCKS)';
+var BUGNUMBER = 467495;
+var summary = 'TCF_FUN_CLOSURE_VS_VAR is necessary';
 var actual = '';
 var expect = '';
 
@@ -51,13 +51,24 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
-  jit(true);
 
-  for each (let x in [function(){}, {}, {}, function(){}, function(){},
-                      function(){}]) { ([<y/> for (y in x)]); }
+  function f(x)
+  {
+    var y = 1;
+    if (Math)
+      function x() { }
+    if (Math)
+      function y() { }
+    return [x, y];
+  }
 
-  jit(false);
+  var r = f(0);
+
+  if (typeof(r[0]) != "function")
+    actual += "Bad r[0]";
+
+  if (typeof(r[1]) != "function")
+    throw "Bad r[1]";
 
   reportCompare(expect, actual, summary);
 
