@@ -135,7 +135,7 @@
 
 TableBackgroundPainter::TableBackgroundData::TableBackgroundData()
   : mFrame(nsnull),
-    mBackground(nsnull),
+    mVisible(PR_FALSE),
     mBorder(nsnull),
     mSynthBorder(nsnull)
 {
@@ -164,7 +164,7 @@ TableBackgroundPainter::TableBackgroundData::Clear()
   mRect.Empty();
   mFrame = nsnull;
   mBorder = nsnull;
-  mBackground = nsnull;
+  mVisible = PR_FALSE;
 }
 
 void
@@ -180,7 +180,7 @@ TableBackgroundPainter::TableBackgroundData::SetData()
 {
   NS_PRECONDITION(mFrame, "null frame");
   if (mFrame->IsVisibleForPainting()) {
-    mBackground = mFrame->GetStyleBackground();
+    mVisible = PR_TRUE;
     mBorder = mFrame->GetStyleBorder();
   }
 }
@@ -197,7 +197,16 @@ inline PRBool
 TableBackgroundPainter::TableBackgroundData::ShouldSetBCBorder()
 {
   /* we only need accurate border data when positioning background images*/
-  return mBackground && !(mBackground->mBackgroundFlags & NS_STYLE_BG_IMAGE_NONE);
+  if (!mVisible) {
+    return PR_FALSE;
+  }
+
+  const nsStyleBackground *bg = mFrame->GetStyleBackground();
+  NS_FOR_VISIBLE_BACKGROUND_LAYERS_BACK_TO_FRONT(i, bg) {
+    if (!bg->mLayers[i].mImage.IsEmpty())
+      return PR_TRUE;
+  }
+  return PR_FALSE;
 }
 
 nsresult
@@ -224,14 +233,16 @@ TableBackgroundPainter::TableBackgroundPainter(nsTableFrame*        aTableFrame,
                                                nsPresContext*       aPresContext,
                                                nsIRenderingContext& aRenderingContext,
                                                const nsRect&        aDirtyRect,
-                                               const nsPoint&       aRenderPt)
+                                               const nsPoint&       aRenderPt,
+                                               PRUint32             aBGPaintFlags)
   : mPresContext(aPresContext),
     mRenderingContext(aRenderingContext),
     mRenderPt(aRenderPt),
     mDirtyRect(aDirtyRect),
     mOrigin(aOrigin),
     mCols(nsnull),
-    mZeroBorder(aPresContext)
+    mZeroBorder(aPresContext),
+    mBGPaintFlags(aBGPaintFlags)
 {
   MOZ_COUNT_CTOR(TableBackgroundPainter);
 
@@ -274,15 +285,13 @@ nsresult
 TableBackgroundPainter::PaintTableFrame(nsTableFrame*         aTableFrame,
                                         nsTableRowGroupFrame* aFirstRowGroup,
                                         nsTableRowGroupFrame* aLastRowGroup,
-                                        nsMargin*             aDeflate)
+                                        const nsMargin&       aDeflate)
 {
   NS_PRECONDITION(aTableFrame, "null frame");
   TableBackgroundData tableData;
   tableData.SetFull(aTableFrame);
   tableData.mRect.MoveTo(0,0); //using table's coords
-  if (aDeflate) {
-    tableData.mRect.Deflate(*aDeflate);
-  }
+  tableData.mRect.Deflate(aDeflate);
   if (mIsBorderCollapse && tableData.ShouldSetBCBorder()) {
     if (aFirstRowGroup && aLastRowGroup && mNumCols > 0) {
       //only handle non-degenerate tables; we need a more robust BC model
@@ -316,9 +325,9 @@ TableBackgroundPainter::PaintTableFrame(nsTableFrame*         aTableFrame,
     nsCSSRendering::PaintBackgroundWithSC(mPresContext, mRenderingContext,
                                           tableData.mFrame, mDirtyRect,
                                           tableData.mRect + mRenderPt,
-                                          *tableData.mBackground,
+                                          tableData.mFrame->GetStyleContext(),
                                           *tableData.mBorder,
-                                          0);
+                                          mBGPaintFlags);
   }
   tableData.Destroy(mPresContext);
   return NS_OK;
@@ -346,8 +355,9 @@ TableBackgroundPainter::TranslateContext(nscoord aDX,
 }
 
 nsresult
-TableBackgroundPainter::PaintTable(nsTableFrame* aTableFrame,
-                                   nsMargin*     aDeflate)
+TableBackgroundPainter::PaintTable(nsTableFrame*   aTableFrame,
+                                   const nsMargin& aDeflate,
+                                   PRBool          aPaintTableBackground)
 {
   NS_PRECONDITION(aTableFrame, "null table frame");
 
@@ -355,13 +365,17 @@ TableBackgroundPainter::PaintTable(nsTableFrame* aTableFrame,
   aTableFrame->OrderRowGroups(rowGroups);
 
   if (rowGroups.Length() < 1) { //degenerate case
-    PaintTableFrame(aTableFrame, nsnull, nsnull, nsnull);
+    if (aPaintTableBackground) {
+      PaintTableFrame(aTableFrame, nsnull, nsnull, nsMargin(0,0,0,0));
+    }
     /* No cells; nothing else to paint */
     return NS_OK;
   }
 
-  PaintTableFrame(aTableFrame, rowGroups[0], rowGroups[rowGroups.Length() - 1],
-                  aDeflate);
+  if (aPaintTableBackground) {
+    PaintTableFrame(aTableFrame, rowGroups[0], rowGroups[rowGroups.Length() - 1],
+                    aDeflate);
+  }
 
   /*Set up column background/border data*/
   if (mNumCols > 0) {
@@ -441,8 +455,7 @@ TableBackgroundPainter::PaintTable(nsTableFrame* aTableFrame,
     // group may not be a child of the table.
     mRowGroup.mRect.MoveTo(rg->GetOffsetTo(aTableFrame));
     if (mRowGroup.mRect.Intersects(mDirtyRect - mRenderPt)) {
-      nsresult rv = PaintRowGroup(rg,
-              rg->IsPseudoStackingContextFromStyle() || rg->IsScrolled());
+      nsresult rv = PaintRowGroup(rg, rg->IsPseudoStackingContextFromStyle());
       if (NS_FAILED(rv)) return rv;
     }
   }
@@ -612,8 +625,8 @@ TableBackgroundPainter::PaintCell(nsTableCellFrame* aCell,
 
   PRInt32 colIndex;
   aCell->GetColIndex(colIndex);
-  NS_ASSERTION(colIndex < mNumCols, "prevent array boundary violation");
-  if (mNumCols <= colIndex)
+  NS_ASSERTION(colIndex < PRInt32(mNumCols), "prevent array boundary violation");
+  if (PRInt32(mNumCols) <= colIndex)
     return NS_OK;
 
   //Paint column group background
@@ -621,9 +634,9 @@ TableBackgroundPainter::PaintCell(nsTableCellFrame* aCell,
     nsCSSRendering::PaintBackgroundWithSC(mPresContext, mRenderingContext,
                                           mCols[colIndex].mColGroup->mFrame, mDirtyRect,
                                           mCols[colIndex].mColGroup->mRect + mRenderPt,
-                                          *mCols[colIndex].mColGroup->mBackground,
+                                          mCols[colIndex].mColGroup->mFrame->GetStyleContext(),
                                           *mCols[colIndex].mColGroup->mBorder,
-                                          0, &mCellRect);
+                                          mBGPaintFlags, &mCellRect);
   }
 
   //Paint column background
@@ -631,9 +644,9 @@ TableBackgroundPainter::PaintCell(nsTableCellFrame* aCell,
     nsCSSRendering::PaintBackgroundWithSC(mPresContext, mRenderingContext,
                                           mCols[colIndex].mCol.mFrame, mDirtyRect,
                                           mCols[colIndex].mCol.mRect + mRenderPt,
-                                          *mCols[colIndex].mCol.mBackground,
+                                          mCols[colIndex].mCol.mFrame->GetStyleContext(),
                                           *mCols[colIndex].mCol.mBorder,
-                                          0, &mCellRect);
+                                          mBGPaintFlags, &mCellRect);
   }
 
   //Paint row group background
@@ -641,8 +654,9 @@ TableBackgroundPainter::PaintCell(nsTableCellFrame* aCell,
     nsCSSRendering::PaintBackgroundWithSC(mPresContext, mRenderingContext,
                                           mRowGroup.mFrame, mDirtyRect,
                                           mRowGroup.mRect + mRenderPt,
-                                          *mRowGroup.mBackground, *mRowGroup.mBorder,
-                                          0, &mCellRect);
+                                          mRowGroup.mFrame->GetStyleContext(),
+                                          *mRowGroup.mBorder,
+                                          mBGPaintFlags, &mCellRect);
   }
 
   //Paint row background
@@ -650,14 +664,15 @@ TableBackgroundPainter::PaintCell(nsTableCellFrame* aCell,
     nsCSSRendering::PaintBackgroundWithSC(mPresContext, mRenderingContext,
                                           mRow.mFrame, mDirtyRect,
                                           mRow.mRect + mRenderPt,
-                                          *mRow.mBackground, *mRow.mBorder,
-                                          0, &mCellRect);
+                                          mRow.mFrame->GetStyleContext(),
+                                          *mRow.mBorder,
+                                          mBGPaintFlags, &mCellRect);
   }
 
   //Paint cell background in border-collapse unless we're just passing
   if (mIsBorderCollapse && !aPassSelf) {
     aCell->PaintCellBackground(mRenderingContext, mDirtyRect,
-                               mCellRect.TopLeft());
+                               mCellRect.TopLeft(), mBGPaintFlags);
   }
 
   return NS_OK;
