@@ -40,6 +40,9 @@
 #include "base/basictypes.h"
 #endif
 
+// FIXME(bug 332648): Give me a real API please!
+#include "jscntxt.h"
+
 #include "nsJSNPRuntime.h"
 #include "nsNPAPIPlugin.h"
 #include "nsNPAPIPluginInstance.h"
@@ -54,9 +57,6 @@
 #include "prmem.h"
 #include "nsIContent.h"
 
-// FIXME(bug 332648): Give me a real API please!
-#include "jscntxt.h"
-
 using namespace mozilla::plugins::parent;
 
 #ifdef MOZ_IPC
@@ -145,23 +145,23 @@ NPClass nsJSObjWrapper::sJSObjWrapperNPClass =
   };
 
 static JSBool
-NPObjWrapper_AddProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp);
+NPObjWrapper_AddProperty(JSContext *cx, JSObject *obj, jsid id, jsval *vp);
 
 static JSBool
-NPObjWrapper_DelProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp);
+NPObjWrapper_DelProperty(JSContext *cx, JSObject *obj, jsid id, jsval *vp);
 
 static JSBool
-NPObjWrapper_SetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp);
+NPObjWrapper_SetProperty(JSContext *cx, JSObject *obj, jsid id, JSBool strict, jsval *vp);
 
 static JSBool
-NPObjWrapper_GetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp);
+NPObjWrapper_GetProperty(JSContext *cx, JSObject *obj, jsid id, jsval *vp);
 
 static JSBool
 NPObjWrapper_newEnumerate(JSContext *cx, JSObject *obj, JSIterateOp enum_op,
                           jsval *statep, jsid *idp);
 
 static JSBool
-NPObjWrapper_NewResolve(JSContext *cx, JSObject *obj, jsval id, uintN flags,
+NPObjWrapper_NewResolve(JSContext *cx, JSObject *obj, jsid id, uintN flags,
                         JSObject **objp);
 
 static JSBool
@@ -171,16 +171,14 @@ static void
 NPObjWrapper_Finalize(JSContext *cx, JSObject *obj);
 
 static JSBool
-NPObjWrapper_Call(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
-                  jsval *rval);
+NPObjWrapper_Call(JSContext *cx, uintN argc, jsval *vp);
 
 static JSBool
-NPObjWrapper_Construct(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
-                       jsval *rval);
+NPObjWrapper_Construct(JSContext *cx, uintN argc, jsval *vp);
 
 static JSBool
 CreateNPObjectMember(NPP npp, JSContext *cx, JSObject *obj, NPObject *npobj,
-                     jsval id, NPVariant* getPropertyResult, jsval *vp);
+                     jsid id, NPVariant* getPropertyResult, jsval *vp);
 
 static JSClass sNPObjectJSWrapperClass =
   {
@@ -197,7 +195,7 @@ static JSClass sNPObjectJSWrapperClass =
 typedef struct NPObjectMemberPrivate {
     JSObject *npobjWrapper;
     jsval fieldValue;
-    jsval methodName;
+    NPIdentifier methodName;
     NPP   npp;
 } NPObjectMemberPrivate;
 
@@ -208,8 +206,7 @@ static void
 NPObjectMember_Finalize(JSContext *cx, JSObject *obj);
 
 static JSBool
-NPObjectMember_Call(JSContext *cx, JSObject *obj, uintN argc,
-                    jsval *argv, jsval *rval);
+NPObjectMember_Call(JSContext *cx, uintN argc, jsval *vp);
 
 static uint32
 NPObjectMember_Mark(JSContext *cx, JSObject *obj, void *arg);
@@ -218,7 +215,7 @@ static JSClass sNPObjectMemberClass =
   {
     "NPObject Ambiguous Member class", JSCLASS_HAS_PRIVATE,
     JS_PropertyStub, JS_PropertyStub,
-    JS_PropertyStub, JS_PropertyStub, JS_EnumerateStub,
+    JS_PropertyStub, JS_StrictPropertyStub, JS_EnumerateStub,
     JS_ResolveStub, NPObjectMember_Convert,
     NPObjectMember_Finalize, nsnull, nsnull, NPObjectMember_Call,
     nsnull, nsnull, nsnull, NPObjectMember_Mark, nsnull
@@ -253,7 +250,7 @@ OnWrapperCreated()
 {
   if (sWrapperCount++ == 0) {
     static const char rtsvc_id[] = "@mozilla.org/js/xpc/RuntimeService;1";
-    nsCOMPtr<nsIJSRuntimeService_MOZILLA_1_9_2> rtsvc(do_GetService(rtsvc_id));
+    nsCOMPtr<nsIJSRuntimeService> rtsvc(do_GetService(rtsvc_id));
     if (!rtsvc)
       return;
 
@@ -457,11 +454,22 @@ JSValToNPVariant(NPP npp, JSContext *cx, jsval val, NPVariant *variant)
     } else if (JSVAL_IS_INT(val)) {
       INT32_TO_NPVARIANT(JSVAL_TO_INT(val), *variant);
     } else if (JSVAL_IS_DOUBLE(val)) {
-      DOUBLE_TO_NPVARIANT(*JSVAL_TO_DOUBLE(val), *variant);
+      jsdouble d = JSVAL_TO_DOUBLE(val);
+      jsint i;
+      if (JS_DoubleIsInt32(d, &i)) {
+        INT32_TO_NPVARIANT(i, *variant);
+      } else {
+        DOUBLE_TO_NPVARIANT(d, *variant);
+      }
     } else if (JSVAL_IS_STRING(val)) {
       JSString *jsstr = JSVAL_TO_STRING(val);
-      nsDependentString str((PRUnichar *)::JS_GetStringChars(jsstr),
-                            ::JS_GetStringLength(jsstr));
+      size_t length;
+      const jschar *chars = ::JS_GetStringCharsZAndLength(cx, jsstr, &length);
+      if (!chars) {
+          return false;
+      }
+
+      nsDependentString str(chars, length);
 
       PRUint32 len;
       char *p = ToNewUTF8String(str, &len);
@@ -583,7 +591,7 @@ nsJSObjWrapper::NP_Invalidate(NPObject *npobj)
 
   if (jsnpobj && jsnpobj->mJSObj) {
     // Unroot the object's JSObject
-    ::JS_RemoveRootRT(sJSRuntime, &jsnpobj->mJSObj);
+    js_RemoveRoot(sJSRuntime, &jsnpobj->mJSObj);
 
     if (sJSObjWrappers.ops) {
       // Remove the wrapper from the hash
@@ -598,31 +606,21 @@ nsJSObjWrapper::NP_Invalidate(NPObject *npobj)
 }
 
 static JSBool
-GetProperty(JSContext *cx, JSObject *obj, NPIdentifier identifier, jsval *rval)
+GetProperty(JSContext *cx, JSObject *obj, NPIdentifier id, jsval *rval)
 {
-  jsval id = (jsval)identifier;
-
-  if (JSVAL_IS_STRING(id)) {
-    JSString *str = JSVAL_TO_STRING(id);
-
-    return ::JS_GetUCProperty(cx, obj, ::JS_GetStringChars(str),
-                              ::JS_GetStringLength(str), rval);
-  }
-
-  NS_ASSERTION(JSVAL_IS_INT(id), "id must be either string or int!\n");
-
-  return ::JS_GetElement(cx, obj, JSVAL_TO_INT(id), rval);
+  NS_ASSERTION(NPIdentifierIsInt(id) || NPIdentifierIsString(id),
+               "id must be either string or int!\n");
+  return ::JS_GetPropertyById(cx, obj, NPIdentifierToJSId(id), rval);
 }
 
 // static
 bool
-nsJSObjWrapper::NP_HasMethod(NPObject *npobj, NPIdentifier identifier)
+nsJSObjWrapper::NP_HasMethod(NPObject *npobj, NPIdentifier id)
 {
   NPP npp = NPPStack::Peek();
   JSContext *cx = GetJSContext(npp);
 
   if (!cx) {
-    NS_ERROR("Null cx in nsJSObjWrapper::NP_HasMethod!");
     return PR_FALSE;
   }
 
@@ -637,10 +635,15 @@ nsJSObjWrapper::NP_HasMethod(NPObject *npobj, NPIdentifier identifier)
 
   AutoCXPusher pusher(cx);
   JSAutoRequest ar(cx);
+  JSAutoEnterCompartment ac;
+
+  if (!ac.enter(cx, npjsobj->mJSObj))
+    return PR_FALSE;
+
   AutoJSExceptionReporter reporter(cx);
 
   jsval v;
-  JSBool ok = GetProperty(cx, npjsobj->mJSObj, identifier, &v);
+  JSBool ok = GetProperty(cx, npjsobj->mJSObj, id, &v);
 
   return ok && !JSVAL_IS_PRIMITIVE(v) &&
     ::JS_ObjectIsFunction(cx, JSVAL_TO_OBJECT(v));
@@ -654,7 +657,6 @@ doInvoke(NPObject *npobj, NPIdentifier method, const NPVariant *args,
   JSContext *cx = GetJSContext(npp);
 
   if (!cx) {
-    NS_ERROR("Null cx in doInvoke!");
     return PR_FALSE;
   }
 
@@ -672,9 +674,14 @@ doInvoke(NPObject *npobj, NPIdentifier method, const NPVariant *args,
 
   AutoCXPusher pusher(cx);
   JSAutoRequest ar(cx);
+  JSAutoEnterCompartment ac;
+
+  if (!ac.enter(cx, npjsobj->mJSObj))
+    return PR_FALSE;
+
   AutoJSExceptionReporter reporter(cx);
 
-  if ((jsval)method != JSVAL_VOID) {
+  if (method != NPIdentifier_VOID) {
     if (!GetProperty(cx, npjsobj->mJSObj, method, &fv) ||
         ::JS_TypeOfValue(cx, fv) != JSTYPE_FUNCTION) {
       return PR_FALSE;
@@ -697,35 +704,33 @@ doInvoke(NPObject *npobj, NPIdentifier method, const NPVariant *args,
     }
   }
 
-  JSTempValueRooter tvr;
-  JS_PUSH_TEMP_ROOT(cx, 0, jsargs, &tvr);
-
-  // Convert args
-  for (PRUint32 i = 0; i < argCount; ++i) {
-    jsargs[i] = NPVariantToJSVal(npp, cx, args + i);
-    ++tvr.count;
-  }
-
   jsval v;
   JSBool ok;
 
-  if (ctorCall) {
-    JSObject *global = ::JS_GetGlobalForObject(cx, npjsobj->mJSObj);
-    JSObject *newObj =
-      ::JS_ConstructObjectWithArguments(cx, JS_GET_CLASS(cx, npjsobj->mJSObj),
-                                        nsnull, global, argCount, jsargs);
+  {
+    js::AutoArrayRooter tvr(cx, 0, jsargs);
 
-    if (newObj) {
-      v = OBJECT_TO_JSVAL(newObj);
-      ok = JS_TRUE;
-    } else {
-      ok = JS_FALSE;
+    // Convert args
+    for (PRUint32 i = 0; i < argCount; ++i) {
+      jsargs[i] = NPVariantToJSVal(npp, cx, args + i);
+      tvr.changeLength(i + 1);
     }
-  } else {
-    ok = ::JS_CallFunctionValue(cx, npjsobj->mJSObj, fv, argCount, jsargs, &v);
-  }
 
-  JS_POP_TEMP_ROOT(cx, &tvr);
+    if (ctorCall) {
+      JSObject *newObj =
+        ::JS_New(cx, npjsobj->mJSObj, argCount, jsargs);
+
+      if (newObj) {
+        v = OBJECT_TO_JSVAL(newObj);
+        ok = JS_TRUE;
+      } else {
+        ok = JS_FALSE;
+      }
+    } else {
+      ok = ::JS_CallFunctionValue(cx, npjsobj->mJSObj, fv, argCount, jsargs, &v);
+    }
+
+  }
 
   if (jsargs != jsargs_buf)
     PR_Free(jsargs);
@@ -744,7 +749,7 @@ nsJSObjWrapper::NP_Invoke(NPObject *npobj, NPIdentifier method,
                           const NPVariant *args, uint32_t argCount,
                           NPVariant *result)
 {
-  if ((jsval)method == JSVAL_VOID) {
+  if (method == NPIdentifier_VOID) {
     return PR_FALSE;
   }
 
@@ -756,19 +761,18 @@ bool
 nsJSObjWrapper::NP_InvokeDefault(NPObject *npobj, const NPVariant *args,
                                  uint32_t argCount, NPVariant *result)
 {
-  return doInvoke(npobj, (NPIdentifier)JSVAL_VOID, args, argCount, PR_FALSE,
+  return doInvoke(npobj, NPIdentifier_VOID, args, argCount, PR_FALSE,
                   result);
 }
 
 // static
 bool
-nsJSObjWrapper::NP_HasProperty(NPObject *npobj, NPIdentifier identifier)
+nsJSObjWrapper::NP_HasProperty(NPObject *npobj, NPIdentifier id)
 {
   NPP npp = NPPStack::Peek();
   JSContext *cx = GetJSContext(npp);
 
   if (!cx) {
-    NS_ERROR("Null cx in nsJSObjWrapper::NP_HasProperty!");
     return PR_FALSE;
   }
 
@@ -780,37 +784,31 @@ nsJSObjWrapper::NP_HasProperty(NPObject *npobj, NPIdentifier identifier)
   }
 
   nsJSObjWrapper *npjsobj = (nsJSObjWrapper *)npobj;
-  jsval id = (jsval)identifier;
   JSBool found, ok = JS_FALSE;
 
   AutoCXPusher pusher(cx);
   JSAutoRequest ar(cx);
   AutoJSExceptionReporter reporter(cx);
+  JSAutoEnterCompartment ac;
 
-  if (JSVAL_IS_STRING(id)) {
-    JSString *str = JSVAL_TO_STRING(id);
-
-    ok = ::JS_HasUCProperty(cx, npjsobj->mJSObj, ::JS_GetStringChars(str),
-                            ::JS_GetStringLength(str), &found);
-  } else {
-    NS_ASSERTION(JSVAL_IS_INT(id), "id must be either string or int!\n");
-
-    ok = ::JS_HasElement(cx, npjsobj->mJSObj, JSVAL_TO_INT(id), &found);
-  }
+  if (!ac.enter(cx, npjsobj->mJSObj))
+    return PR_FALSE;
 
+  NS_ASSERTION(NPIdentifierIsInt(id) || NPIdentifierIsString(id),
+               "id must be either string or int!\n");
+  ok = ::JS_HasPropertyById(cx, npjsobj->mJSObj, NPIdentifierToJSId(id), &found);
   return ok && found;
 }
 
 // static
 bool
-nsJSObjWrapper::NP_GetProperty(NPObject *npobj, NPIdentifier identifier,
+nsJSObjWrapper::NP_GetProperty(NPObject *npobj, NPIdentifier id,
                                NPVariant *result)
 {
   NPP npp = NPPStack::Peek();
   JSContext *cx = GetJSContext(npp);
 
   if (!cx) {
-    NS_ERROR("Null cx in nsJSObjWrapper::NP_GetProperty!");
     return PR_FALSE;
   }
 
@@ -826,22 +824,25 @@ nsJSObjWrapper::NP_GetProperty(NPObject *npobj, NPIdentifier identifier,
   AutoCXPusher pusher(cx);
   JSAutoRequest ar(cx);
   AutoJSExceptionReporter reporter(cx);
+  JSAutoEnterCompartment ac;
+
+  if (!ac.enter(cx, npjsobj->mJSObj))
+    return PR_FALSE;
 
   jsval v;
-  return (GetProperty(cx, npjsobj->mJSObj, identifier, &v) &&
+  return (GetProperty(cx, npjsobj->mJSObj, id, &v) &&
           JSValToNPVariant(npp, cx, v, result));
 }
 
 // static
 bool
-nsJSObjWrapper::NP_SetProperty(NPObject *npobj, NPIdentifier identifier,
+nsJSObjWrapper::NP_SetProperty(NPObject *npobj, NPIdentifier id,
                                const NPVariant *value)
 {
   NPP npp = NPPStack::Peek();
   JSContext *cx = GetJSContext(npp);
 
   if (!cx) {
-    NS_ERROR("Null cx in nsJSObjWrapper::NP_SetProperty!");
     return PR_FALSE;
   }
 
@@ -853,26 +854,22 @@ nsJSObjWrapper::NP_SetProperty(NPObject *npobj, NPIdentifier identifier,
   }
 
   nsJSObjWrapper *npjsobj = (nsJSObjWrapper *)npobj;
-  jsval id = (jsval)identifier;
   JSBool ok = JS_FALSE;
 
   AutoCXPusher pusher(cx);
   JSAutoRequest ar(cx);
   AutoJSExceptionReporter reporter(cx);
+  JSAutoEnterCompartment ac;
+
+  if (!ac.enter(cx, npjsobj->mJSObj))
+    return PR_FALSE;
 
   jsval v = NPVariantToJSVal(npp, cx, value);
-  JSAutoTempValueRooter tvr(cx, v);
+  js::AutoValueRooter tvr(cx, v);
 
-  if (JSVAL_IS_STRING(id)) {
-    JSString *str = JSVAL_TO_STRING(id);
-
-    ok = ::JS_SetUCProperty(cx, npjsobj->mJSObj, ::JS_GetStringChars(str),
-                            ::JS_GetStringLength(str), &v);
-  } else {
-    NS_ASSERTION(JSVAL_IS_INT(id), "id must be either string or int!\n");
-
-    ok = ::JS_SetElement(cx, npjsobj->mJSObj, JSVAL_TO_INT(id), &v);
-  }
+  NS_ASSERTION(NPIdentifierIsInt(id) || NPIdentifierIsString(id),
+               "id must be either string or int!\n");
+  ok = ::JS_SetPropertyById(cx, npjsobj->mJSObj, NPIdentifierToJSId(id), &v);
 
   // return ok == JS_TRUE to quiet down compiler warning, even if
   // return ok is what we really want.
@@ -881,13 +878,12 @@ nsJSObjWrapper::NP_SetProperty(NPObject *npobj, NPIdentifier identifier,
 
 // static
 bool
-nsJSObjWrapper::NP_RemoveProperty(NPObject *npobj, NPIdentifier identifier)
+nsJSObjWrapper::NP_RemoveProperty(NPObject *npobj, NPIdentifier id)
 {
   NPP npp = NPPStack::Peek();
   JSContext *cx = GetJSContext(npp);
 
   if (!cx) {
-    NS_ERROR("Null cx in nsJSObjWrapper::NP_RemoveProperty!");
     return PR_FALSE;
   }
 
@@ -899,53 +895,32 @@ nsJSObjWrapper::NP_RemoveProperty(NPObject *npobj, NPIdentifier identifier)
   }
 
   nsJSObjWrapper *npjsobj = (nsJSObjWrapper *)npobj;
-  jsval id = (jsval)identifier;
   JSBool ok = JS_FALSE;
 
   AutoCXPusher pusher(cx);
   JSAutoRequest ar(cx);
   AutoJSExceptionReporter reporter(cx);
   jsval deleted = JSVAL_FALSE;
+  JSAutoEnterCompartment ac;
 
-  if (JSVAL_IS_STRING(id)) {
-    JSString *str = JSVAL_TO_STRING(id);
+  if (!ac.enter(cx, npjsobj->mJSObj))
+    return PR_FALSE;
 
-    ok = ::JS_DeleteUCProperty2(cx, npjsobj->mJSObj, ::JS_GetStringChars(str),
-                                ::JS_GetStringLength(str), &deleted);
+  NS_ASSERTION(NPIdentifierIsInt(id) || NPIdentifierIsString(id),
+               "id must be either string or int!\n");
+  ok = ::JS_DeletePropertyById2(cx, npjsobj->mJSObj, NPIdentifierToJSId(id), &deleted);
+  if (ok && deleted == JSVAL_TRUE) {
+    // FIXME: See bug 425823, we shouldn't need to do this, and once
+    // that bug is fixed we can remove this code.
 
-    if (ok && deleted) {
-      // FIXME: See bug 425823, we shouldn't need to do this, and once
-      // that bug is fixed we can remove this code.
+    JSBool hasProp;
+    ok = ::JS_HasPropertyById(cx, npjsobj->mJSObj, NPIdentifierToJSId(id), &hasProp);
 
-      JSBool hasProp;
-      ok = ::JS_HasUCProperty(cx, npjsobj->mJSObj, ::JS_GetStringChars(str),
-                              ::JS_GetStringLength(str), &hasProp);
+    if (ok && hasProp) {
+      // The property might have been deleted, but it got
+      // re-resolved, so no, it's not really deleted.
 
-      if (ok && hasProp) {
-        // The property might have been deleted, but it got
-        // re-resolved, so no, it's not really deleted.
-
-        deleted = JSVAL_FALSE;
-      }
-    }
-  } else {
-    NS_ASSERTION(JSVAL_IS_INT(id), "id must be either string or int!\n");
-
-    ok = ::JS_DeleteElement2(cx, npjsobj->mJSObj, JSVAL_TO_INT(id), &deleted);
-
-    if (ok && deleted) {
-      // FIXME: See bug 425823, we shouldn't need to do this, and once
-      // that bug is fixed we can remove this code.
-
-      JSBool hasProp;
-      ok = ::JS_HasElement(cx, npjsobj->mJSObj, JSVAL_TO_INT(id), &hasProp);
-
-      if (ok && hasProp) {
-        // The property might have been deleted, but it got
-        // re-resolved, so no, it's not really deleted.
-
-        deleted = JSVAL_FALSE;
-      }
+      deleted = JSVAL_FALSE;
     }
   }
 
@@ -956,17 +931,16 @@ nsJSObjWrapper::NP_RemoveProperty(NPObject *npobj, NPIdentifier identifier)
 
 //static
 bool
-nsJSObjWrapper::NP_Enumerate(NPObject *npobj, NPIdentifier **identifier,
+nsJSObjWrapper::NP_Enumerate(NPObject *npobj, NPIdentifier **idarray,
                              uint32_t *count)
 {
   NPP npp = NPPStack::Peek();
   JSContext *cx = GetJSContext(npp);
 
-  *identifier = 0;
+  *idarray = 0;
   *count = 0;
 
   if (!cx) {
-    NS_ERROR("Null cx in nsJSObjWrapper::NP_Enumerate!");
     return PR_FALSE;
   }
 
@@ -982,6 +956,10 @@ nsJSObjWrapper::NP_Enumerate(NPObject *npobj, NPIdentifier **identifier,
   AutoCXPusher pusher(cx);
   JSAutoRequest ar(cx);
   AutoJSExceptionReporter reporter(cx);
+  JSAutoEnterCompartment ac;
+
+  if (!ac.enter(cx, npjsobj->mJSObj))
+    return PR_FALSE;
 
   JSIdArray *ida = ::JS_Enumerate(cx, npjsobj->mJSObj);
   if (!ida) {
@@ -989,8 +967,8 @@ nsJSObjWrapper::NP_Enumerate(NPObject *npobj, NPIdentifier **identifier,
   }
 
   *count = ida->length;
-  *identifier = (NPIdentifier *)PR_Malloc(*count * sizeof(NPIdentifier));
-  if (!*identifier) {
+  *idarray = (NPIdentifier *)PR_Malloc(*count * sizeof(NPIdentifier));
+  if (!*idarray) {
     ThrowJSException(cx, "Memory allocation failed for NPIdentifier!");
 
     ::JS_DestroyIdArray(cx, ida);
@@ -1002,26 +980,27 @@ nsJSObjWrapper::NP_Enumerate(NPObject *npobj, NPIdentifier **identifier,
     jsval v;
     if (!::JS_IdToValue(cx, ida->vector[i], &v)) {
       ::JS_DestroyIdArray(cx, ida);
-      PR_Free(*identifier);
+      PR_Free(*idarray);
       return PR_FALSE;
     }
 
+    NPIdentifier id;
     if (JSVAL_IS_STRING(v)) {
-      JSString *str = JSVAL_TO_STRING(v);
+      JSString *str = JS_InternJSString(cx, JSVAL_TO_STRING(v));
+      if (!str) {
+          ::JS_DestroyIdArray(cx, ida);
+          PR_Free(*idarray);
 
-      if (!JS_InternUCStringN(cx, ::JS_GetStringChars(str),
-                              ::JS_GetStringLength(str))) {
-        ::JS_DestroyIdArray(cx, ida);
-        PR_Free(*identifier);
-
-        return PR_FALSE;
+          return PR_FALSE;
       }
+      id = StringToNPIdentifier(str);
     } else {
       NS_ASSERTION(JSVAL_IS_INT(v),
                    "The element in ida must be either string or int!\n");
+      id = IntToNPIdentifier(JSVAL_TO_INT(v));
     }
 
-    (*identifier)[i] = (NPIdentifier)v;
+    (*idarray)[i] = id;
   }
 
   ::JS_DestroyIdArray(cx, ida);
@@ -1034,8 +1013,7 @@ bool
 nsJSObjWrapper::NP_Construct(NPObject *npobj, const NPVariant *args,
                              uint32_t argCount, NPVariant *result)
 {
-  return doInvoke(npobj, (NPIdentifier)JSVAL_VOID, args, argCount, PR_TRUE,
-                  result);
+  return doInvoke(npobj, NPIdentifier_VOID, args, argCount, PR_TRUE, result);
 }
 
 
@@ -1084,13 +1062,16 @@ nsJSObjWrapper::GetNewOrUsed(NPP npp, JSContext *cx, JSObject *obj)
     cx = GetJSContext(npp);
 
     if (!cx) {
-      NS_ERROR("Unable to find a JSContext in "
-               "nsJSObjWrapper::GetNewOrUsed()!");
+      NS_ERROR("Unable to find a JSContext in nsJSObjWrapper::GetNewOrUsed()!");
 
       return nsnull;
     }
   }
 
+  // No need to enter the right compartment here as we only get the
+  // class and private from the JSObject, neither of which cares about
+  // compartments.
+
   JSClass *clazz = JS_GET_CLASS(cx, obj);
 
   if (clazz == &sNPObjectJSWrapperClass) {
@@ -1164,7 +1145,7 @@ nsJSObjWrapper::GetNewOrUsed(NPP npp, JSContext *cx, JSObject *obj)
 
   // Root the JSObject, its lifetime is now tied to that of the
   // NPObject.
-  if (!::JS_AddNamedRoot(cx, &wrapper->mJSObj, "nsJSObjWrapper::mJSObject")) {
+  if (!::JS_AddNamedObjectRoot(cx, &wrapper->mJSObj, "nsJSObjWrapper::mJSObject")) {
     NS_ERROR("Failed to root JSObject!");
 
     _releaseobject(wrapper);
@@ -1195,7 +1176,7 @@ GetNPObject(JSContext *cx, JSObject *obj)
 // Does not actually add a property because this is always followed by a
 // SetProperty call.
 static JSBool
-NPObjWrapper_AddProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
+NPObjWrapper_AddProperty(JSContext *cx, JSObject *obj, jsid id, jsval *vp)
 {
   NPObject *npobj = GetNPObject(cx, obj);
 
@@ -1212,7 +1193,8 @@ NPObjWrapper_AddProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
 
   PluginDestructionGuard pdg(LookupNPP(npobj));
 
-  JSBool hasProperty = npobj->_class->hasProperty(npobj, (NPIdentifier)id);
+  NPIdentifier identifier = JSIdToNPIdentifier(id);
+  JSBool hasProperty = npobj->_class->hasProperty(npobj, identifier);
   if (!ReportExceptionIfPending(cx))
     return JS_FALSE;
 
@@ -1221,7 +1203,7 @@ NPObjWrapper_AddProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
 
   // We must permit methods here since JS_DefineUCFunction() will add
   // the function as a property
-  JSBool hasMethod = npobj->_class->hasMethod(npobj, (NPIdentifier)id);
+  JSBool hasMethod = npobj->_class->hasMethod(npobj, identifier);
   if (!ReportExceptionIfPending(cx))
     return JS_FALSE;
 
@@ -1235,7 +1217,7 @@ NPObjWrapper_AddProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
 }
 
 static JSBool
-NPObjWrapper_DelProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
+NPObjWrapper_DelProperty(JSContext *cx, JSObject *obj, jsid id, jsval *vp)
 {
   NPObject *npobj = GetNPObject(cx, obj);
 
@@ -1248,8 +1230,10 @@ NPObjWrapper_DelProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
 
   PluginDestructionGuard pdg(LookupNPP(npobj));
 
+  NPIdentifier identifier = JSIdToNPIdentifier(id);
+
   if (!NPObjectIsOutOfProcessProxy(npobj)) {
-    JSBool hasProperty = npobj->_class->hasProperty(npobj, (NPIdentifier)id);
+    JSBool hasProperty = npobj->_class->hasProperty(npobj, identifier);
     if (!ReportExceptionIfPending(cx))
       return JS_FALSE;
 
@@ -1257,14 +1241,14 @@ NPObjWrapper_DelProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
       return JS_TRUE;
   }
 
-  if (!npobj->_class->removeProperty(npobj, (NPIdentifier)id))
+  if (!npobj->_class->removeProperty(npobj, identifier))
     *vp = JSVAL_FALSE;
 
   return ReportExceptionIfPending(cx);
 }
 
 static JSBool
-NPObjWrapper_SetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
+NPObjWrapper_SetProperty(JSContext *cx, JSObject *obj, jsid id, JSBool strict, jsval *vp)
 {
   NPObject *npobj = GetNPObject(cx, obj);
 
@@ -1287,8 +1271,10 @@ NPObjWrapper_SetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
 
   PluginDestructionGuard pdg(npp);
 
+  NPIdentifier identifier = JSIdToNPIdentifier(id);
+
   if (!NPObjectIsOutOfProcessProxy(npobj)) {
-    JSBool hasProperty = npobj->_class->hasProperty(npobj, (NPIdentifier)id);
+    JSBool hasProperty = npobj->_class->hasProperty(npobj, identifier);
     if (!ReportExceptionIfPending(cx))
       return JS_FALSE;
 
@@ -1306,7 +1292,7 @@ NPObjWrapper_SetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
     return JS_FALSE;
   }
 
-  JSBool ok = npobj->_class->setProperty(npobj, (NPIdentifier)id, &npv);
+  JSBool ok = npobj->_class->setProperty(npobj, identifier, &npv);
   _releasevariantvalue(&npv); // Release the variant
   if (!ReportExceptionIfPending(cx))
     return JS_FALSE;
@@ -1321,7 +1307,7 @@ NPObjWrapper_SetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
 }
 
 static JSBool
-NPObjWrapper_GetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
+NPObjWrapper_GetProperty(JSContext *cx, JSObject *obj, jsid id, jsval *vp)
 {
   NPObject *npobj = GetNPObject(cx, obj);
 
@@ -1348,6 +1334,8 @@ NPObjWrapper_GetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
   NPVariant npv;
   VOID_TO_NPVARIANT(npv);
 
+  NPIdentifier identifier = JSIdToNPIdentifier(id);
+
 #ifdef MOZ_IPC
   if (NPObjectIsOutOfProcessProxy(npobj)) {
     PluginScriptableObjectParent* actor =
@@ -1357,7 +1345,7 @@ NPObjWrapper_GetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
     if (!actor)
       return JS_FALSE;
 
-    JSBool success = actor->GetPropertyHelper((NPIdentifier)id, &hasProperty,
+    JSBool success = actor->GetPropertyHelper(identifier, &hasProperty,
                                               &hasMethod, &npv);
     if (!ReportExceptionIfPending(cx)) {
       if (success)
@@ -1382,11 +1370,11 @@ NPObjWrapper_GetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
   }
 #endif
 
-  hasProperty = npobj->_class->hasProperty(npobj, (NPIdentifier)id);
+  hasProperty = npobj->_class->hasProperty(npobj, identifier);
   if (!ReportExceptionIfPending(cx))
     return JS_FALSE;
 
-  hasMethod = npobj->_class->hasMethod(npobj, (NPIdentifier)id);
+  hasMethod = npobj->_class->hasMethod(npobj, identifier);
   if (!ReportExceptionIfPending(cx))
     return JS_FALSE;
 
@@ -1395,7 +1383,7 @@ NPObjWrapper_GetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
     return CreateNPObjectMember(npp, cx, obj, npobj, id, nsnull, vp);
 
   if (hasProperty) {
-    if (npobj->_class->getProperty(npobj, (NPIdentifier)id, &npv))
+    if (npobj->_class->getProperty(npobj, identifier, &npv))
       *vp = NPVariantToJSVal(npp, cx, &npv);
 
     _releasevariantvalue(&npv);
@@ -1423,7 +1411,7 @@ CallNPMethodInternal(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
 
   NPObject *npobj = (NPObject *)::JS_GetPrivate(cx, obj);
 
-  if (!npobj || !npobj->_class || !npobj->_class->invoke) {
+  if (!npobj || !npobj->_class) {
     ThrowJSException(cx, "Bad NPObject as private data!");
 
     return JS_FALSE;
@@ -1495,10 +1483,10 @@ CallNPMethodInternal(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
 
     if (npobj->_class->invoke) {
       JSFunction *fun = (JSFunction *)::JS_GetPrivate(cx, funobj);
-      jsval method = STRING_TO_JSVAL(::JS_GetFunctionId(fun));
+      JSString *name = ::JS_GetFunctionId(fun);
+      NPIdentifier id = StringToNPIdentifier(name);
 
-      ok = npobj->_class->invoke(npobj, (NPIdentifier)method, npargs, argc,
-                                 &v);
+      ok = npobj->_class->invoke(npobj, id, npargs, argc, &v);
     } else {
       ok = JS_FALSE;
 
@@ -1545,10 +1533,13 @@ CallNPMethodInternal(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
 }
 
 static JSBool
-CallNPMethod(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
-             jsval *rval)
+CallNPMethod(JSContext *cx, uintN argc, jsval *vp)
 {
-  return CallNPMethodInternal(cx, obj, argc, argv, rval, PR_FALSE);
+  JSObject *obj = JS_THIS_OBJECT(cx, vp);
+  if (!obj)
+      return JS_FALSE;
+
+  return CallNPMethodInternal(cx, obj, argc, JS_ARGV(cx, vp), vp, PR_FALSE);
 }
 
 struct NPObjectEnumerateState {
@@ -1577,6 +1568,7 @@ NPObjWrapper_newEnumerate(JSContext *cx, JSObject *obj, JSIterateOp enum_op,
 
   switch(enum_op) {
   case JSENUMERATE_INIT:
+  case JSENUMERATE_INIT_ALL:
     state = new NPObjectEnumerateState();
     if (!state) {
       ThrowJSException(cx, "Memory allocation failed for "
@@ -1607,7 +1599,7 @@ NPObjWrapper_newEnumerate(JSContext *cx, JSObject *obj, JSIterateOp enum_op,
     state->index = 0;
     *statep = PRIVATE_TO_JSVAL(state);
     if (idp) {
-      *idp = INT_TO_JSVAL(length);
+      *idp = INT_TO_JSID(length);
     }
 
     break;
@@ -1617,7 +1609,8 @@ NPObjWrapper_newEnumerate(JSContext *cx, JSObject *obj, JSIterateOp enum_op,
     enum_value = state->value;
     length = state->length;
     if (state->index != length) {
-      return ::JS_ValueToId(cx, (jsval)enum_value[state->index++], idp);
+      *idp = NPIdentifierToJSId(enum_value[state->index++]);
+      return JS_TRUE;
     }
 
     // FALL THROUGH
@@ -1636,7 +1629,7 @@ NPObjWrapper_newEnumerate(JSContext *cx, JSObject *obj, JSIterateOp enum_op,
 }
 
 static JSBool
-NPObjWrapper_NewResolve(JSContext *cx, JSObject *obj, jsval id, uintN flags,
+NPObjWrapper_NewResolve(JSContext *cx, JSObject *obj, jsid id, uintN flags,
                         JSObject **objp)
 {
   NPObject *npobj = GetNPObject(cx, obj);
@@ -1650,26 +1643,18 @@ NPObjWrapper_NewResolve(JSContext *cx, JSObject *obj, jsval id, uintN flags,
 
   PluginDestructionGuard pdg(LookupNPP(npobj));
 
-  PRBool hasProperty = npobj->_class->hasProperty(npobj, (NPIdentifier)id);
+  NPIdentifier identifier = JSIdToNPIdentifier(id);
+
+  PRBool hasProperty = npobj->_class->hasProperty(npobj, identifier);
   if (!ReportExceptionIfPending(cx))
     return JS_FALSE;
 
   if (hasProperty) {
-    JSBool ok;
-
-    if (JSVAL_IS_STRING(id)) {
-      JSString *str = JSVAL_TO_STRING(id);
-
-      ok = ::JS_DefineUCProperty(cx, obj, ::JS_GetStringChars(str),
-                                 ::JS_GetStringLength(str), JSVAL_VOID, nsnull,
-                                 nsnull, JSPROP_ENUMERATE);
-    } else {
-      ok = ::JS_DefineElement(cx, obj, JSVAL_TO_INT(id), JSVAL_VOID, nsnull,
-                              nsnull, JSPROP_ENUMERATE);
-    }
-
-    if (!ok) {
-      return JS_FALSE;
+    NS_ASSERTION(JSID_IS_STRING(id) || JSID_IS_INT(id),
+                 "id must be either string or int!\n");
+    if (!::JS_DefinePropertyById(cx, obj, id, JSVAL_VOID, nsnull,
+                                 nsnull, JSPROP_ENUMERATE)) {
+        return JS_FALSE;
     }
 
     *objp = obj;
@@ -1677,31 +1662,16 @@ NPObjWrapper_NewResolve(JSContext *cx, JSObject *obj, jsval id, uintN flags,
     return JS_TRUE;
   }
 
-  PRBool hasMethod = npobj->_class->hasMethod(npobj, (NPIdentifier)id);
+  PRBool hasMethod = npobj->_class->hasMethod(npobj, identifier);
   if (!ReportExceptionIfPending(cx))
     return JS_FALSE;
 
   if (hasMethod) {
-    JSString *str = nsnull;
+    NS_ASSERTION(JSID_IS_STRING(id) || JSID_IS_INT(id),
+                 "id must be either string or int!\n");
 
-    if (JSVAL_IS_STRING(id)) {
-      str = JSVAL_TO_STRING(id);
-    } else {
-      NS_ASSERTION(JSVAL_IS_INT(id), "id must be either string or int!\n");
-
-      str = ::JS_ValueToString(cx, id);
-
-      if (!str) {
-        // OOM. The JS engine throws exceptions for us in this case.
-
-        return JS_FALSE;
-      }
-    }
-
-    JSFunction *fnc =
-      ::JS_DefineUCFunction(cx, obj, ::JS_GetStringChars(str),
-                            ::JS_GetStringLength(str), CallNPMethod, 0,
-                            JSPROP_ENUMERATE);
+    JSFunction *fnc = ::JS_DefineFunctionById(cx, obj, id, CallNPMethod, 0,
+                                              JSPROP_ENUMERATE);
 
     *objp = obj;
 
@@ -1740,19 +1710,17 @@ NPObjWrapper_Finalize(JSContext *cx, JSObject *obj)
 }
 
 static JSBool
-NPObjWrapper_Call(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
-                  jsval *rval)
+NPObjWrapper_Call(JSContext *cx, uintN argc, jsval *vp)
 {
-  return CallNPMethodInternal(cx, JSVAL_TO_OBJECT(argv[-2]), argc, argv, rval,
-                              PR_FALSE);
+  return CallNPMethodInternal(cx, JSVAL_TO_OBJECT(JS_CALLEE(cx, vp)), argc,
+                              JS_ARGV(cx, vp), vp, PR_FALSE);
 }
 
 static JSBool
-NPObjWrapper_Construct(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
-                       jsval *rval)
+NPObjWrapper_Construct(JSContext *cx, uintN argc, jsval *vp)
 {
-  return CallNPMethodInternal(cx, JSVAL_TO_OBJECT(argv[-2]), argc, argv, rval,
-                              PR_TRUE);
+  return CallNPMethodInternal(cx, JSVAL_TO_OBJECT(JS_CALLEE(cx, vp)), argc,
+                              JS_ARGV(cx, vp), vp, PR_TRUE);
 }
 
 class NPObjWrapperHashEntry : public PLDHashEntryHdr
@@ -1975,7 +1943,10 @@ NPObjWrapperPluginDestroyedCallback(PLDHashTable *table, PLDHashEntryHdr *hdr,
       PR_Free(npobj);
     }
 
-    ::JS_SetPrivate(nppcx->cx, entry->mJSObj, nsnull);
+    JSAutoEnterCompartment ac;
+    if (ac.enter(nppcx->cx, entry->mJSObj)) {
+      ::JS_SetPrivate(nppcx->cx, entry->mJSObj, nsnull);
+    }
 
     table->ops = ops;    
 
@@ -2066,14 +2037,23 @@ nsJSNPRuntime::OnPluginDestroy(NPP npp)
     return;
   }
 
-  nsCOMPtr<nsINode> node(do_QueryInterface(element));
-
-  JSObject *obj;
-  if (!node || !(obj = node->GetWrapper())) {
+  nsCOMPtr<nsIXPConnectWrappedNative> holder;
+  xpc->GetWrappedNativeOfNativeObject(cx, sgo->GetGlobalJSObject(), content,
+                                      NS_GET_IID(nsISupports),
+                                      getter_AddRefs(holder));
+  if (!holder) {
     return;
   }
 
-  JSObject *proto;
+  JSObject *obj, *proto;
+  holder->GetJSObject(&obj);
+
+  JSAutoEnterCompartment ac;
+
+  if (obj && !ac.enter(cx, obj)) {
+    // Failure to enter compartment, nothing more we can do then.
+    return;
+  }
 
   // Loop over the DOM element's JS object prototype chain and remove
   // all JS objects of the class sNPObjectJSWrapperClass (there should
@@ -2117,7 +2097,7 @@ LookupNPP(NPObject *npobj)
 
 JSBool
 CreateNPObjectMember(NPP npp, JSContext *cx, JSObject *obj, NPObject* npobj,
-                     jsval id,  NPVariant* getPropertyResult, jsval *vp)
+                     jsid id,  NPVariant* getPropertyResult, jsval *vp)
 {
   NS_ENSURE_TRUE(vp, JS_FALSE);
 
@@ -2144,10 +2124,12 @@ CreateNPObjectMember(NPP npp, JSContext *cx, JSObject *obj, NPObject* npobj,
   }
 
   *vp = OBJECT_TO_JSVAL(memobj);
-  ::JS_AddRoot(cx, vp);
+  ::JS_AddValueRoot(cx, vp);
 
   ::JS_SetPrivate(cx, memobj, (void *)memberPrivate);
 
+  NPIdentifier identifier = JSIdToNPIdentifier(id);
+
   jsval fieldValue;
   NPVariant npv;
   NPBool hasProperty;
@@ -2160,15 +2142,15 @@ CreateNPObjectMember(NPP npp, JSContext *cx, JSObject *obj, NPObject* npobj,
   else {
     VOID_TO_NPVARIANT(npv);
 
-    NPBool hasProperty = npobj->_class->getProperty(npobj, (NPIdentifier)id,
+    NPBool hasProperty = npobj->_class->getProperty(npobj, identifier,
                                                     &npv);
     if (!ReportExceptionIfPending(cx)) {
-      ::JS_RemoveRoot(cx, vp);
+      ::JS_RemoveValueRoot(cx, vp);
       return JS_FALSE;
     }
 
     if (!hasProperty) {
-      ::JS_RemoveRoot(cx, vp);
+      ::JS_RemoveValueRoot(cx, vp);
       return JS_FALSE;
     }
   }
@@ -2185,10 +2167,10 @@ CreateNPObjectMember(NPP npp, JSContext *cx, JSObject *obj, NPObject* npobj,
   memberPrivate->npobjWrapper = obj;
 
   memberPrivate->fieldValue = fieldValue;
-  memberPrivate->methodName = id;
+  memberPrivate->methodName = identifier;
   memberPrivate->npp = npp;
 
-  ::JS_RemoveRoot(cx, vp);
+  ::JS_RemoveValueRoot(cx, vp);
 
   return JS_TRUE;
 }
@@ -2235,16 +2217,15 @@ NPObjectMember_Finalize(JSContext *cx, JSObject *obj)
 }
 
 static JSBool
-NPObjectMember_Call(JSContext *cx, JSObject *obj,
-                    uintN argc, jsval *argv, jsval *rval)
+NPObjectMember_Call(JSContext *cx, uintN argc, jsval *vp)
 {
-  JSObject *memobj = JSVAL_TO_OBJECT(argv[-2]);
+  JSObject *memobj = JSVAL_TO_OBJECT(JS_CALLEE(cx, vp));
   NS_ENSURE_TRUE(memobj, JS_FALSE);
 
   NPObjectMemberPrivate *memberPrivate =
     (NPObjectMemberPrivate *)::JS_GetInstancePrivate(cx, memobj,
                                                      &sNPObjectMemberClass,
-                                                     argv);
+                                                     JS_ARGV(cx, vp));
   if (!memberPrivate || !memberPrivate->npobjWrapper)
     return JS_FALSE;
 
@@ -2272,6 +2253,7 @@ NPObjectMember_Call(JSContext *cx, JSObject *obj,
 
   // Convert arguments
   PRUint32 i;
+  jsval *argv = JS_ARGV(cx, vp);
   for (i = 0; i < argc; ++i) {
     if (!JSValToNPVariant(memberPrivate->npp, cx, argv[i], npargs + i)) {
       ThrowJSException(cx, "Error converting jsvals to NPVariants!");
@@ -2284,9 +2266,10 @@ NPObjectMember_Call(JSContext *cx, JSObject *obj,
     }
   }
 
+
   NPVariant npv;
   JSBool ok;
-  ok = npobj->_class->invoke(npobj, (NPIdentifier)memberPrivate->methodName,
+  ok = npobj->_class->invoke(npobj, memberPrivate->methodName,
                              npargs, argc, &npv);
 
   // Release arguments.
@@ -2307,9 +2290,9 @@ NPObjectMember_Call(JSContext *cx, JSObject *obj,
     return JS_FALSE;
   }
 
-  *rval = NPVariantToJSVal(memberPrivate->npp, cx, &npv);
+  JS_SET_RVAL(cx, vp, NPVariantToJSVal(memberPrivate->npp, cx, &npv));
 
-  // *rval now owns the value, release our reference.
+  // *vp now owns the value, release our reference.
   _releasevariantvalue(&npv);
 
   return ReportExceptionIfPending(cx);
@@ -2326,7 +2309,7 @@ NPObjectMember_Mark(JSContext *cx, JSObject *obj, void *arg)
     return 0;
 
   if (!JSVAL_IS_PRIMITIVE(memberPrivate->fieldValue)) {
-    ::JS_MarkGCThing(cx, JSVAL_TO_OBJECT(memberPrivate->fieldValue),
+    ::JS_MarkGCThing(cx, memberPrivate->fieldValue,
                      "NPObject Member => fieldValue", arg);
   }
 
@@ -2334,7 +2317,7 @@ NPObjectMember_Mark(JSContext *cx, JSObject *obj, void *arg)
   // NPObject, so make sure to mark the NPObject wrapper to keep the
   // NPObject alive as long as this NPObjectMember is alive.
   if (memberPrivate->npobjWrapper) {
-    ::JS_MarkGCThing(cx, memberPrivate->npobjWrapper,
+    ::JS_MarkGCThing(cx, OBJECT_TO_JSVAL(memberPrivate->npobjWrapper),
                      "NPObject Member => npobjWrapper", arg);
   }
 
