@@ -23,8 +23,9 @@ function testOnLoad() {
   var sstring = Cc["@mozilla.org/supports-string;1"].
                 createInstance(Ci.nsISupportsString);
   sstring.data = location.search;
+
   ww.openWindow(window, "chrome://mochikit/content/browser-harness.xul", "browserTest",
-                "chrome,centerscreen,dialog,resizable,titlebar,toolbar=no,width=800,height=600", sstring);
+                "chrome,centerscreen,dialog=no,resizable,titlebar,toolbar=no,width=800,height=600", sstring);
 }
 
 function Tester(aTests, aDumper, aCallback) {
@@ -33,14 +34,19 @@ function Tester(aTests, aDumper, aCallback) {
   this.callback = aCallback;
   this._cs = Cc["@mozilla.org/consoleservice;1"].
              getService(Ci.nsIConsoleService);
+  this._wm = Cc["@mozilla.org/appshell/window-mediator;1"].
+             getService(Ci.nsIWindowMediator);
+  this._fm = Cc["@mozilla.org/focus-manager;1"].
+             getService(Ci.nsIFocusManager);
 
-  var scriptLoader = Cc["@mozilla.org/moz/jssubscript-loader;1"].
-                     getService(Ci.mozIJSSubScriptLoader);
-  scriptLoader.loadSubScript("chrome://mochikit/content/tests/SimpleTest/EventUtils.js", this.EventUtils);
+  this._scriptLoader = Cc["@mozilla.org/moz/jssubscript-loader;1"].
+                       getService(Ci.mozIJSSubScriptLoader);
+  this._scriptLoader.loadSubScript("chrome://mochikit/content/tests/SimpleTest/EventUtils.js", this.EventUtils);
   // Avoid polluting this scope with packed.js contents.
   var simpleTestScope = {};
-  scriptLoader.loadSubScript("chrome://mochikit/content/MochiKit/packed.js", simpleTestScope);
-  scriptLoader.loadSubScript("chrome://mochikit/content/tests/SimpleTest/SimpleTest.js", simpleTestScope);
+  this._scriptLoader.loadSubScript("chrome://mochikit/content/MochiKit/packed.js", simpleTestScope);
+  this._scriptLoader.loadSubScript("chrome://mochikit/content/tests/SimpleTest/SimpleTest.js", simpleTestScope);
+  this._scriptLoader.loadSubScript("chrome://mochikit/content/chrome-harness.js", simpleTestScope);
   this.SimpleTest = simpleTestScope.SimpleTest;
 }
 Tester.prototype = {
@@ -49,31 +55,68 @@ Tester.prototype = {
 
   checker: null,
   currentTestIndex: -1,
+  lastStartTime: null,
   get currentTest() {
     return this.tests[this.currentTestIndex];
   },
   get done() {
     return this.currentTestIndex == this.tests.length - 1;
   },
-  step: function Tester_step() {
-    this.currentTestIndex++;
-  },
 
   start: function Tester_start() {
     this.dumper.dump("*** Start BrowserChrome Test Results ***\n");
     this._cs.registerListener(this);
 
     if (this.tests.length)
-      this.execTest();
+      this.nextTest();
     else
       this.finish();
   },
 
+  waitForWindowsState: function Tester_waitForWindowsState(aCallback) {
+    let timedOut = this.currentTest && this.currentTest.timedOut;
+    let baseMsg = timedOut ? "Found a {elt} after previous test timed out"
+                           : this.currentTest ? "Found an unexpected {elt} at the end of test run"
+                                              : "Found an unexpected {elt}";
+
+    if (this.currentTest && window.gBrowser && gBrowser.tabs.length > 1) {
+      while (gBrowser.tabs.length > 1) {
+        let lastTab = gBrowser.tabContainer.lastChild;
+        let msg = baseMsg.replace("{elt}", "tab") +
+                  ": " + lastTab.linkedBrowser.currentURI.spec;
+        this.currentTest.addResult(new testResult(false, msg, "", false));
+        gBrowser.removeTab(lastTab);
+      }
+    }
+
+    this.dumper.dump("TEST-INFO | checking window state\n");
+    let windowsEnum = this._wm.getEnumerator("navigator:browser");
+    while (windowsEnum.hasMoreElements()) {
+      let win = windowsEnum.getNext();
+      if (win != window && !win.closed) {
+        let msg = baseMsg.replace("{elt}", "browser window");
+        if (this.currentTest)
+          this.currentTest.addResult(new testResult(false, msg, "", false));
+        else
+          this.dumper.dump("TEST-UNEXPECTED-FAIL | (browser-test.js) | " + msg + "\n");
+
+        win.close();
+      }
+    }
+
+    // Make sure the window is raised before each test.
+    let self = this;
+    this.SimpleTest.waitForFocus(function() {
+      aCallback.apply(self);
+    });
+  },
+
   finish: function Tester_finish(aSkipSummary) {
     this._cs.unregisterListener(this);
 
+    this.dumper.dump("\nINFO TEST-START | Shutdown\n");
     if (this.tests.length) {
-      this.dumper.dump("\nBrowser Chrome Test Summary\n");
+      this.dumper.dump("Browser Chrome Test Summary\n");
 
       function sum(a,b) a+b;
       var passCount = this.tests.map(function (f) f.passCount).reduce(sum);
@@ -99,35 +142,62 @@ Tester.prototype = {
   },
 
   observe: function Tester_observe(aConsoleMessage) {
-    var msg = "Console message: " + aConsoleMessage.message;
-    this.currentTest.addResult(new testMessage(msg));
+    try {
+      var msg = "Console message: " + aConsoleMessage.message;
+      if (this.currentTest)
+        this.currentTest.addResult(new testMessage(msg));
+      else
+        this.dumper.dump("TEST-INFO | (browser-test.js) | " + msg);
+    } catch (ex) {
+      // Swallow exception so we don't lead to another error being reported,
+      // throwing us into an infinite loop
+    }
   },
 
-  execTest: function Tester_execTest() {
+  nextTest: function Tester_nextTest() {
+    if (this.currentTest) {
+      // Run cleanup functions for the current test before moving on to the
+      // next one.
+      let testScope = this.currentTest.scope;
+      while (testScope.__cleanupFunctions.length > 0) {
+        let func = testScope.__cleanupFunctions.shift();
+        try {
+          func.apply(testScope);
+        }
+        catch (ex) {
+          this.currentTest.addResult(new testResult(false, "Cleanup function threw an exception", ex, false));
+        }
+      };
+
+      // Clear document.popupNode.  The test could have set it to a custom value
+      // for its own purposes, nulling it out it will go back to the default
+      // behavior of returning the last opened popup.
+      document.popupNode = null;
+
+      // Note the test run time
+      let time = Date.now() - this.lastStartTime;
+      this.dumper.dump("INFO TEST-END | " + this.currentTest.path + " | finished in " + time + "ms\n");
+      this.currentTest.setDuration(time);
+    }
+
+    // Check the window state for the current test before moving to the next one.
+    // This also causes us to check before starting any tests, since nextTest()
+    // is invoked to start the tests.
+    this.waitForWindowsState(this.realNextTest);
+  },
+
+  realNextTest: function Test_realNextTest() {
     if (this.done) {
       this.finish();
       return;
     }
 
-    // Make sure the window is raised before each test.
-    let fm = Cc["@mozilla.org/focus-manager;1"].getService(Ci.nsIFocusManager);
-    if (fm.activeWindow != window) {
-      this.dumper.dump("Waiting for window activation...\n");
-      let self = this;
-      window.addEventListener("activate", function () {
-        window.removeEventListener("activate", arguments.callee, false);
-        setTimeout(function () {
-          self.execTest();
-        }, 0);
-      }, false);
-      window.focus();
-      return;
-    }
+    this.currentTestIndex++;
+    this.execTest();
+  },
 
-    // Move to the next test (or first test).
-    this.step();
-
-    this.dumper.dump("Running " + this.currentTest.path + "...\n");
+  execTest: function Tester_execTest() {
+    this.dumper.dump("TEST-START | " + this.currentTest.path + "\n");
 
     // Load the tests into a testscope
     this.currentTest.scope = new testScope(this, this.currentTest);
@@ -135,13 +205,33 @@ Tester.prototype = {
     // Import utils in the test scope.
     this.currentTest.scope.EventUtils = this.EventUtils;
     this.currentTest.scope.SimpleTest = this.SimpleTest;
+    this.currentTest.scope.gTestPath = this.currentTest.path;
 
-    var scriptLoader = Cc["@mozilla.org/moz/jssubscript-loader;1"].
-                       getService(Ci.mozIJSSubScriptLoader);
+    // Override SimpleTest methods with ours.
+    ["ok", "is", "isnot", "todo", "todo_is", "todo_isnot"].forEach(function(m) {
+      this.SimpleTest[m] = this[m];
+    }, this.currentTest.scope);
+
+    //load the tools to work with chrome .jar and remote
     try {
-      scriptLoader.loadSubScript(this.currentTest.path, this.currentTest.scope);
+      this._scriptLoader.loadSubScript("chrome://mochikit/content/chrome-harness.js", this.currentTest.scope);
+    } catch (ex) { /* no chrome-harness tools */ }
+
+    // Import head.js script if it exists.
+    var currentTestDirPath =
+      this.currentTest.path.substr(0, this.currentTest.path.lastIndexOf("/"));
+    var headPath = currentTestDirPath + "/head.js";
+    try {
+      this._scriptLoader.loadSubScript(headPath, this.currentTest.scope);
+    } catch (ex) { /* no head */ }
+
+    // Import the test script.
+    try {
+      this._scriptLoader.loadSubScript(this.currentTest.path,
+                                       this.currentTest.scope);
 
       // Run the test
+      this.lastStartTime = Date.now();
       this.currentTest.scope.test();
     } catch (ex) {
       this.currentTest.addResult(new testResult(false, "Exception thrown", ex, false));
@@ -151,14 +241,24 @@ Tester.prototype = {
     // If the test ran synchronously, move to the next test, otherwise the test
     // will trigger the next test when it is done.
     if (this.currentTest.scope.__done) {
-      this.execTest();
+      this.nextTest();
     }
     else {
       var self = this;
       this.currentTest.scope.__waitTimer = setTimeout(function() {
-        self.currentTest.addResult(new testResult(false, "Timed out", "", false));
+        if (--self.currentTest.scope.__timeoutFactor > 0) {
+          // We were asked to wait a bit longer.
+          self.currentTest.scope.info(
+            "Longer timeout required, waiting longer...  Remaining timeouts: " +
+            self.currentTest.scope.__timeoutFactor);
+          self.currentTest.scope.__waitTimer =
+            setTimeout(arguments.callee, TIMEOUT_SECONDS * 1000);
+          return;
+        }
+        self.currentTest.addResult(new testResult(false, "Test timed out", "", false));
+        self.currentTest.timedOut = true;
         self.currentTest.scope.__waitTimer = null;
-        self.execTest();
+        self.nextTest();
       }, TIMEOUT_SECONDS * 1000);
     }
   },
@@ -244,12 +344,28 @@ function testScope(aTester, aTest) {
     }, Ci.nsIThread.DISPATCH_NORMAL);
   };
 
-  this.waitForExplicitFinish = function test_WFEF() {
+  this.waitForExplicitFinish = function test_waitForExplicitFinish() {
     self.__done = false;
   };
 
-  this.waitForFocus = function (callback, targetWindow) {
-    self.SimpleTest.waitForFocus(callback, targetWindow);
+  this.waitForFocus = function test_waitForFocus(callback, targetWindow, expectBlankPage) {
+    self.SimpleTest.waitForFocus(callback, targetWindow, expectBlankPage);
+  };
+
+  this.waitForClipboard = function test_waitForClipboard(expected, setup, success, failure) {
+    self.SimpleTest.waitForClipboard(expected, setup, success, failure);
+  };
+
+  this.registerCleanupFunction = function test_registerCleanupFunction(aFunction) {
+    self.__cleanupFunctions.push(aFunction);
+  };
+
+  this.requestLongerTimeout = function test_requestLongerTimeout(aFactor) {
+    self.__timeoutFactor = aFactor;
+  };
+
+  this.copyToProfile = function test_copyToProfile(filename) {
+    self.SimpleTest.copyToProfile(filename);
   };
 
   this.finish = function test_finish() {
@@ -259,7 +375,7 @@ function testScope(aTester, aTest) {
         if (self.__done && self.__waitTimer) {
           clearTimeout(self.__waitTimer);
           self.__waitTimer = null;
-          self.__tester.execTest();
+          self.__tester.nextTest();
         }
       });
     }
@@ -268,6 +384,8 @@ function testScope(aTester, aTest) {
 testScope.prototype = {
   __done: true,
   __waitTimer: null,
+  __cleanupFunctions: [],
+  __timeoutFactor: 1,
 
   EventUtils: {},
   SimpleTest: {}
