@@ -39,7 +39,8 @@
 
 #include "nsFormFillController.h"
 
-#include "nsStorageFormHistory.h"
+#include "nsIFormAutoComplete.h"
+#include "nsIInputListAutoComplete.h"
 #include "nsIAutoCompleteSimpleResult.h"
 #include "nsString.h"
 #include "nsReadableUtils.h"
@@ -56,7 +57,7 @@
 #include "nsIDOMCompositionListener.h"
 #include "nsIDOMDocument.h"
 #include "nsIDOMElement.h"
-#include "nsIDOMNSHTMLInputElement.h"
+#include "nsIFormControl.h"
 #include "nsIDocument.h"
 #include "nsIContent.h"
 #include "nsIPresShell.h"
@@ -66,10 +67,12 @@
 #include "nsIDOMHTMLFormElement.h"
 #include "nsILoginManager.h"
 #include "nsIDOMMouseEvent.h"
-#include "nsIGenericFactory.h"
+#include "mozilla/ModuleUtils.h"
 #include "nsToolkitCompsCID.h"
 #include "nsEmbedCID.h"
 #include "nsIDOMNSEditableElement.h"
+#include "nsIDOMNSEvent.h"
+#include "mozilla/dom/Element.h"
 
 NS_INTERFACE_MAP_BEGIN(nsFormFillController)
   NS_INTERFACE_MAP_ENTRY(nsIFormFillController)
@@ -81,6 +84,7 @@ NS_INTERFACE_MAP_BEGIN(nsFormFillController)
   NS_INTERFACE_MAP_ENTRY(nsIDOMMouseListener)
   NS_INTERFACE_MAP_ENTRY(nsIDOMCompositionListener)
   NS_INTERFACE_MAP_ENTRY(nsIDOMContextMenuListener)
+  NS_INTERFACE_MAP_ENTRY(nsIMutationObserver)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIFormFillController)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsIDOMEventListener, nsIDOMFocusListener)
 NS_INTERFACE_MAP_END
@@ -117,6 +121,79 @@ nsFormFillController::~nsFormFillController()
   }
 }
 
+////////////////////////////////////////////////////////////////////////
+//// nsIMutationObserver
+//
+
+void
+nsFormFillController::AttributeChanged(nsIDocument* aDocument,
+                                       mozilla::dom::Element* aElement,
+                                       PRInt32 aNameSpaceID,
+                                       nsIAtom* aAttribute, PRInt32 aModType)
+{
+  RevalidateDataList();
+}
+
+void
+nsFormFillController::ContentAppended(nsIDocument* aDocument,
+                                      nsIContent* aContainer,
+                                      nsIContent* aChild,
+                                      PRInt32 aIndexInContainer)
+{
+  RevalidateDataList();
+}
+
+void
+nsFormFillController::ContentInserted(nsIDocument* aDocument,
+                                      nsIContent* aContainer,
+                                      nsIContent* aChild,
+                                      PRInt32 aIndexInContainer)
+{
+  RevalidateDataList();
+}
+
+void
+nsFormFillController::ContentRemoved(nsIDocument* aDocument,
+                                     nsIContent* aContainer,
+                                     nsIContent* aChild,
+                                     PRInt32 aIndexInContainer,
+                                     nsIContent* aPreviousSibling)
+{
+  RevalidateDataList();
+}
+
+void
+nsFormFillController::CharacterDataWillChange(nsIDocument* aDocument,
+                                              nsIContent* aContent,
+                                              CharacterDataChangeInfo* aInfo)
+{
+}
+
+void
+nsFormFillController::CharacterDataChanged(nsIDocument* aDocument,
+                                           nsIContent* aContent,
+                                           CharacterDataChangeInfo* aInfo)
+{
+}
+
+void
+nsFormFillController::AttributeWillChange(nsIDocument* aDocument,
+                                          mozilla::dom::Element* aElement,
+                                          PRInt32 aNameSpaceID,
+                                          nsIAtom* aAttribute, PRInt32 aModType)
+{
+}
+
+void
+nsFormFillController::ParentChainChanged(nsIContent* aContent)
+{
+}
+
+void
+nsFormFillController::NodeWillBeDestroyed(const nsINode* aNode)
+{
+}
+
 ////////////////////////////////////////////////////////////////////////
 //// nsIFormFillController
 
@@ -197,6 +274,8 @@ nsFormFillController::GetPopupOpen(PRBool *aPopupOpen)
 {
   if (mFocusedPopup)
     mFocusedPopup->GetPopupOpen(aPopupOpen);
+  else
+    *aPopupOpen = PR_FALSE;
   return NS_OK;
 }
 
@@ -215,7 +294,8 @@ nsFormFillController::SetPopupOpen(PRBool aPopupOpen)
       NS_ENSURE_STATE(presShell);
       presShell->ScrollContentIntoView(content,
                                        NS_PRESSHELL_SCROLL_IF_NOT_VISIBLE,
-                                       NS_PRESSHELL_SCROLL_IF_NOT_VISIBLE);
+                                       NS_PRESSHELL_SCROLL_IF_NOT_VISIBLE,
+                                       nsIPresShell::SCROLL_OVERFLOW_HIDDEN);
       // mFocusedPopup can be destroyed after ScrollContentIntoView, see bug 420089
       if (mFocusedPopup)
         mFocusedPopup->OpenAutocompletePopup(this, mFocusedInput);
@@ -407,27 +487,24 @@ nsFormFillController::SetTextValue(const nsAString & aTextValue)
 NS_IMETHODIMP
 nsFormFillController::GetSelectionStart(PRInt32 *aSelectionStart)
 {
-  nsCOMPtr<nsIDOMNSHTMLInputElement> input = do_QueryInterface(mFocusedInput);
-  if (input)
-    input->GetSelectionStart(aSelectionStart);
+  if (mFocusedInput)
+    mFocusedInput->GetSelectionStart(aSelectionStart);
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsFormFillController::GetSelectionEnd(PRInt32 *aSelectionEnd)
 {
-  nsCOMPtr<nsIDOMNSHTMLInputElement> input = do_QueryInterface(mFocusedInput);
-  if (input)
-    input->GetSelectionEnd(aSelectionEnd);
+  if (mFocusedInput)
+    mFocusedInput->GetSelectionEnd(aSelectionEnd);
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsFormFillController::SelectTextRange(PRInt32 aStartIndex, PRInt32 aEndIndex)
 {
-  nsCOMPtr<nsIDOMNSHTMLInputElement> input = do_QueryInterface(mFocusedInput);
-  if (input)
-    input->SetSelectionRange(aStartIndex, aEndIndex);
+ if (mFocusedInput)
+    mFocusedInput->SetSelectionRange(aStartIndex, aEndIndex);
   return NS_OK;
 }
 
@@ -496,6 +573,7 @@ NS_IMETHODIMP
 nsFormFillController::StartSearch(const nsAString &aSearchString, const nsAString &aSearchParam,
                                   nsIAutoCompleteResult *aPreviousResult, nsIAutoCompleteObserver *aListener)
 {
+  nsresult rv;
   nsCOMPtr<nsIAutoCompleteResult> result;
 
   // If the login manager has indicated it's responsible for this field, let it
@@ -504,28 +582,98 @@ nsFormFillController::StartSearch(const nsAString &aSearchString, const nsAStrin
   if (mPwmgrInputs.Get(mFocusedInput, &dummy)) {
     // XXX aPreviousResult shouldn't ever be a historyResult type, since we're not letting
     // satchel manage the field?
-    mLoginManager->AutoCompleteSearch(aSearchString,
+    rv = mLoginManager->AutoCompleteSearch(aSearchString,
                                          aPreviousResult,
                                          mFocusedInput,
                                          getter_AddRefs(result));
   } else {
-    nsCOMPtr<nsIAutoCompleteSimpleResult> historyResult;
-    historyResult = do_QueryInterface(aPreviousResult);
+    nsCOMPtr<nsIAutoCompleteResult> formHistoryResult;
+    if (!IsInputAutoCompleteOff()) {
+      nsCOMPtr <nsIFormAutoComplete> formAutoComplete =
+        do_GetService("@mozilla.org/satchel/form-autocomplete;1", &rv);
+      NS_ENSURE_SUCCESS(rv, rv);
 
-    nsFormHistory *history = nsFormHistory::GetInstance();
-    if (history) {
-      history->AutoCompleteSearch(aSearchParam,
-                                  aSearchString,
-                                  historyResult,
-                                  getter_AddRefs(result));
+      rv = formAutoComplete->AutoCompleteSearch(aSearchParam,
+                                                aSearchString,
+                                                mFocusedInput,
+                                                aPreviousResult,
+                                                getter_AddRefs(formHistoryResult));
+
+      NS_ENSURE_SUCCESS(rv, rv);
+    }
+
+    mLastSearchResult = formHistoryResult;
+    mLastListener = aListener;
+    mLastSearchString = aSearchString;
+
+    nsCOMPtr <nsIInputListAutoComplete> inputListAutoComplete =
+      do_GetService("@mozilla.org/satchel/inputlist-autocomplete;1", &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv = inputListAutoComplete->AutoCompleteSearch(formHistoryResult,
+                                                   aSearchString,
+                                                   mFocusedInput,
+                                                   getter_AddRefs(result));
+
+    if (mFocusedInput) {
+      nsCOMPtr<nsIDOMHTMLElement> list;
+      mFocusedInput->GetList(getter_AddRefs(list));
+
+      nsCOMPtr<nsINode> node = do_QueryInterface(list);
+      if(node) {
+        node->AddMutationObserverUnlessExists(this);
+      }
     }
   }
+  NS_ENSURE_SUCCESS(rv, rv);
 
   aListener->OnSearchResult(this, result);  
   
   return NS_OK;
 }
 
+class UpdateSearchResultRunnable : public nsRunnable
+{
+public:
+  UpdateSearchResultRunnable(nsIAutoCompleteObserver* aObserver,
+                             nsIAutoCompleteSearch* aSearch,
+                             nsIAutoCompleteResult* aResult)
+    : mObserver(aObserver)
+    , mSearch(aSearch)
+    , mResult(aResult)
+  {}
+
+  NS_IMETHOD Run() {
+    NS_ASSERTION(mObserver, "You shouldn't call this runnable with a null observer!");
+
+    mObserver->OnUpdateSearchResult(mSearch, mResult);
+    return NS_OK;
+  }
+
+private:
+  nsCOMPtr<nsIAutoCompleteObserver> mObserver;
+  nsCOMPtr<nsIAutoCompleteSearch> mSearch;
+  nsCOMPtr<nsIAutoCompleteResult> mResult;
+};
+
+void nsFormFillController::RevalidateDataList()
+{
+  nsresult rv;
+  nsCOMPtr <nsIInputListAutoComplete> inputListAutoComplete =
+    do_GetService("@mozilla.org/satchel/inputlist-autocomplete;1", &rv);
+
+  nsCOMPtr<nsIAutoCompleteResult> result;
+
+  rv = inputListAutoComplete->AutoCompleteSearch(mLastSearchResult,
+                                                 mLastSearchString,
+                                                 mFocusedInput,
+                                                 getter_AddRefs(result));
+
+  nsCOMPtr<nsIRunnable> event =
+    new UpdateSearchResultRunnable(mLastListener, this, result);
+  NS_DispatchToCurrentThread(event);
+}
+
 NS_IMETHODIMP
 nsFormFillController::StopSearch()
 {
@@ -538,6 +686,10 @@ nsFormFillController::StopSearch()
 NS_IMETHODIMP
 nsFormFillController::HandleEvent(nsIDOMEvent* aEvent)
 {
+  // Drop untrusted events from content
+  if (!IsEventTrusted(aEvent))
+    return NS_OK;
+
   nsAutoString type;
   aEvent->GetType(type);
 
@@ -585,45 +737,69 @@ nsFormFillController::RemoveForDOMDocumentEnumerator(nsISupports* aKey,
 NS_IMETHODIMP
 nsFormFillController::Focus(nsIDOMEvent* aEvent)
 {
+  // Drop untrusted events from content
+  if (!IsEventTrusted(aEvent))
+    return NS_OK;
+
   nsCOMPtr<nsIDOMEventTarget> target;
   aEvent->GetTarget(getter_AddRefs(target));
   
   nsCOMPtr<nsIDOMHTMLInputElement> input = do_QueryInterface(target);
   if (!input)
     return NS_OK;
-    
-    nsAutoString type;
-    input->GetType(type);
 
-    PRBool isReadOnly = PR_FALSE;
-    input->GetReadOnly(&isReadOnly);
-                                  
-    nsAutoString autocomplete; 
-    input->GetAttribute(NS_LITERAL_STRING("autocomplete"), autocomplete);
+  PRBool isReadOnly = PR_FALSE;
+  input->GetReadOnly(&isReadOnly);
 
-    PRInt32 dummy;
-    PRBool isPwmgrInput = PR_FALSE;
-    if (mPwmgrInputs.Get(input, &dummy))
-        isPwmgrInput = PR_TRUE;
+  nsAutoString autocomplete;
+  input->GetAttribute(NS_LITERAL_STRING("autocomplete"), autocomplete);
 
-    if (type.LowerCaseEqualsLiteral("text") && !isReadOnly &&
-        (!autocomplete.LowerCaseEqualsLiteral("off") || isPwmgrInput)) {
+  PRInt32 dummy;
+  PRBool isPwmgrInput = PR_FALSE;
+  if (mPwmgrInputs.Get(input, &dummy))
+      isPwmgrInput = PR_TRUE;
 
-      nsCOMPtr<nsIDOMHTMLFormElement> form;
-      input->GetForm(getter_AddRefs(form));
-      if (form)
-        form->GetAttribute(NS_LITERAL_STRING("autocomplete"), autocomplete);
-
-      if (!form || !autocomplete.LowerCaseEqualsLiteral("off") || isPwmgrInput)
-        StartControllingInput(input);
-    }
+  nsCOMPtr<nsIFormControl> formControl = do_QueryInterface(input);
+  if (formControl && formControl->IsSingleLineTextControl(PR_TRUE) &&
+      !isReadOnly || isPwmgrInput) {
+    StartControllingInput(input);
+  }
     
   return NS_OK;
 }
 
+PRBool 
+nsFormFillController::IsInputAutoCompleteOff()
+{
+  PRBool autoCompleteOff = PR_FALSE;
+
+  if (mFocusedInput) {
+    nsAutoString autocomplete; 
+    mFocusedInput->GetAttribute(NS_LITERAL_STRING("autocomplete"), autocomplete);
+    
+    // Check the input for autocomplete="off", then the form
+    if (autocomplete.LowerCaseEqualsLiteral("off")) {
+      autoCompleteOff = PR_TRUE;
+    } else {
+
+      nsCOMPtr<nsIDOMHTMLFormElement> form;
+      mFocusedInput->GetForm(getter_AddRefs(form));
+      if (form)
+        form->GetAttribute(NS_LITERAL_STRING("autocomplete"), autocomplete);
+      autoCompleteOff = autocomplete.LowerCaseEqualsLiteral("off");
+    }
+  }
+
+  return autoCompleteOff;
+}
+
 NS_IMETHODIMP
 nsFormFillController::Blur(nsIDOMEvent* aEvent)
 {
+  // Drop untrusted events from content
+  if (!IsEventTrusted(aEvent))
+    return NS_OK;
+
   if (mFocusedInput)
     StopControllingInput();
   
@@ -636,18 +812,30 @@ nsFormFillController::Blur(nsIDOMEvent* aEvent)
 NS_IMETHODIMP
 nsFormFillController::KeyDown(nsIDOMEvent* aEvent)
 {
+  // Drop untrusted events from content
+  if (!IsEventTrusted(aEvent))
+    return NS_OK;
+
   return NS_OK;
 } 
 
 NS_IMETHODIMP
 nsFormFillController::KeyUp(nsIDOMEvent* aEvent)
 {
+  // Drop untrusted events from content
+  if (!IsEventTrusted(aEvent))
+    return NS_OK;
+
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsFormFillController::KeyPress(nsIDOMEvent* aEvent)
 {
+  // Drop untrusted events from content
+  if (!IsEventTrusted(aEvent))
+    return NS_OK;
+
   NS_ASSERTION(mController, "should have a controller!");
   if (!mFocusedInput || !mController)
     return NS_OK;
@@ -666,7 +854,7 @@ nsFormFillController::KeyPress(nsIDOMEvent* aEvent)
     mController->HandleDelete(&cancel);
     break;
   case nsIDOMKeyEvent::DOM_VK_BACK_SPACE:
-    mController->HandleText(PR_FALSE);
+    mController->HandleText();
     break;
 #else
   case nsIDOMKeyEvent::DOM_VK_BACK_SPACE:
@@ -677,7 +865,7 @@ nsFormFillController::KeyPress(nsIDOMEvent* aEvent)
       if (isShift)
         mController->HandleDelete(&cancel);
       else
-        mController->HandleText(PR_FALSE);
+        mController->HandleText();
 
       break;
     }
@@ -725,6 +913,10 @@ nsFormFillController::KeyPress(nsIDOMEvent* aEvent)
 NS_IMETHODIMP
 nsFormFillController::HandleStartComposition(nsIDOMEvent* aCompositionEvent)
 {
+  // Drop untrusted events from content
+  if (!IsEventTrusted(aCompositionEvent))
+    return NS_OK;
+
   NS_ASSERTION(mController, "should have a controller!");
 
   if (mController && mFocusedInput)
@@ -736,6 +928,10 @@ nsFormFillController::HandleStartComposition(nsIDOMEvent* aCompositionEvent)
 NS_IMETHODIMP
 nsFormFillController::HandleEndComposition(nsIDOMEvent* aCompositionEvent)
 {
+  // Drop untrusted events from content
+  if (!IsEventTrusted(aCompositionEvent))
+    return NS_OK;
+
   NS_ASSERTION(mController, "should have a controller!");
 
   if (mController && mFocusedInput)
@@ -744,18 +940,16 @@ nsFormFillController::HandleEndComposition(nsIDOMEvent* aCompositionEvent)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsFormFillController::HandleQueryComposition(nsIDOMEvent* aCompositionEvent)
-{
-  return NS_OK;
-}
-
 ////////////////////////////////////////////////////////////////////////
 //// nsIDOMFormListener
 
 NS_IMETHODIMP
 nsFormFillController::Submit(nsIDOMEvent* aEvent)
 {
+  // Drop untrusted events from content
+  if (!IsEventTrusted(aEvent))
+    return NS_OK;
+
   if (mFocusedInput)
     StopControllingInput();
 
@@ -765,28 +959,44 @@ nsFormFillController::Submit(nsIDOMEvent* aEvent)
 NS_IMETHODIMP
 nsFormFillController::Reset(nsIDOMEvent* aEvent)
 {
+  // Drop untrusted events from content
+  if (!IsEventTrusted(aEvent))
+    return NS_OK;
+
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsFormFillController::Change(nsIDOMEvent* aEvent)
 {
+  // Drop untrusted events from content
+  if (!IsEventTrusted(aEvent))
+    return NS_OK;
+
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsFormFillController::Select(nsIDOMEvent* aEvent)
 {
+  // Drop untrusted events from content
+  if (!IsEventTrusted(aEvent))
+    return NS_OK;
+
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsFormFillController::Input(nsIDOMEvent* aEvent)
 {
+  // Drop untrusted events from content
+  if (!IsEventTrusted(aEvent))
+    return NS_OK;
+
   if (mSuppressOnInput || !mController || !mFocusedInput)
     return NS_OK;
 
-  return mController->HandleText(PR_FALSE);
+  return mController->HandleText();
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -795,6 +1005,10 @@ nsFormFillController::Input(nsIDOMEvent* aEvent)
 NS_IMETHODIMP
 nsFormFillController::MouseDown(nsIDOMEvent* aMouseEvent)
 {
+  // Drop untrusted events from content
+  if (!IsEventTrusted(aMouseEvent))
+    return NS_OK;
+
   if (!mFocusedInput)
     return NS_OK;
 
@@ -828,7 +1042,7 @@ nsFormFillController::MouseDown(nsIDOMEvent* aMouseEvent)
   if (value.Length() > 0) {
     // Show the popup with a filtered result set
     mController->SetSearchString(EmptyString());
-    mController->HandleText(PR_TRUE);
+    mController->HandleText();
   } else {
     // Show the popup with the complete result set.  Can't use HandleText()
     // because it doesn't display the popup if the input is blank.
@@ -842,30 +1056,50 @@ nsFormFillController::MouseDown(nsIDOMEvent* aMouseEvent)
 NS_IMETHODIMP
 nsFormFillController::MouseUp(nsIDOMEvent* aMouseEvent)
 {
+  // Drop untrusted events from content
+  if (!IsEventTrusted(aMouseEvent))
+    return NS_OK;
+
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsFormFillController::MouseClick(nsIDOMEvent* aMouseEvent)
 {
+  // Drop untrusted events from content
+  if (!IsEventTrusted(aMouseEvent))
+    return NS_OK;
+
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsFormFillController::MouseDblClick(nsIDOMEvent* aMouseEvent)
 { 
+  // Drop untrusted events from content
+  if (!IsEventTrusted(aMouseEvent))
     return NS_OK;
+
+  return NS_OK;
 }
 
 NS_IMETHODIMP
 nsFormFillController::MouseOver(nsIDOMEvent* aMouseEvent)
 {
+  // Drop untrusted events from content
+  if (!IsEventTrusted(aMouseEvent))
+    return NS_OK;
+
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsFormFillController::MouseOut(nsIDOMEvent* aMouseEvent)
 {
+  // Drop untrusted events from content
+  if (!IsEventTrusted(aMouseEvent))
+    return NS_OK;
+
   return NS_OK;
 }
 
@@ -874,6 +1108,10 @@ nsFormFillController::MouseOut(nsIDOMEvent* aMouseEvent)
 NS_IMETHODIMP
 nsFormFillController::ContextMenu(nsIDOMEvent* aContextMenuEvent)
 {
+  // Drop untrusted events from content
+  if (!IsEventTrusted(aContextMenuEvent))
+    return NS_OK;
+
   if (mFocusedPopup)
     mFocusedPopup->ClosePopup();
   return NS_OK;
@@ -933,6 +1171,9 @@ nsFormFillController::AddWindowListeners(nsIDOMWindow *aWindow)
   target->AddEventListener(NS_LITERAL_STRING("contextmenu"),
                            static_cast<nsIDOMContextMenuListener *>(this),
                            PR_TRUE);
+
+  // Note that any additional listeners added should ensure that they ignore
+  // untrusted events, which might be sent by content that's up to no good.
 }
 
 void
@@ -1047,6 +1288,16 @@ nsFormFillController::StopControllingInput()
 {
   RemoveKeyListener();
 
+  if(mFocusedInput) {
+    nsCOMPtr<nsIDOMHTMLElement> list;
+    mFocusedInput->GetList(getter_AddRefs(list));
+
+    nsCOMPtr<nsINode> node = do_QueryInterface(list);
+    if (node) {
+      node->RemoveMutationObserver(this);
+    }
+  }
+
   // Reset the controller's input, but not if it has been switched
   // to another input already, which might happen if the user switches
   // focus by clicking another autocomplete textbox
@@ -1114,36 +1365,41 @@ nsFormFillController::GetIndexOfDocShell(nsIDocShell *aDocShell)
   return -1;
 }
 
-NS_GENERIC_FACTORY_CONSTRUCTOR_INIT(nsFormHistory, Init)
-NS_GENERIC_FACTORY_CONSTRUCTOR(nsFormFillController)
-#ifdef MOZ_MORKREADER
-NS_GENERIC_FACTORY_CONSTRUCTOR(nsFormHistoryImporter)
-#endif
-
-static const nsModuleComponentInfo components[] =
+PRBool
+nsFormFillController::IsEventTrusted(nsIDOMEvent *aEvent)
 {
-  { "HTML Form History",
-    NS_FORMHISTORY_CID, 
-    NS_FORMHISTORY_CONTRACTID,
-    nsFormHistoryConstructor },
+  nsresult rv;
 
-  { "HTML Form Fill Controller",
-    NS_FORMFILLCONTROLLER_CID, 
-    "@mozilla.org/satchel/form-fill-controller;1",
-    nsFormFillControllerConstructor },
+  nsCOMPtr<nsIDOMNSEvent> nsevent = do_QueryInterface(aEvent);
+  if (!nsevent)
+    return PR_FALSE;
 
-  { "HTML Form History AutoComplete",
-    NS_FORMFILLCONTROLLER_CID, 
-    NS_FORMHISTORYAUTOCOMPLETE_CONTRACTID,
-    nsFormFillControllerConstructor },
+  PRBool isTrusted;
+  rv = nsevent->GetIsTrusted(&isTrusted);
+  NS_ENSURE_SUCCESS(rv, PR_FALSE);
+  return isTrusted;
+}
 
-#ifdef MOZ_MORKREADER
-  { "Form History Importer",
-    NS_FORMHISTORYIMPORTER_CID,
-    NS_FORMHISTORYIMPORTER_CONTRACTID,
-    nsFormHistoryImporterConstructor },
-#endif
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsFormFillController)
+
+NS_DEFINE_NAMED_CID(NS_FORMFILLCONTROLLER_CID);
+
+static const mozilla::Module::CIDEntry kSatchelCIDs[] = {
+  { &kNS_FORMFILLCONTROLLER_CID, false, NULL, nsFormFillControllerConstructor },
+  { NULL }
 };
 
-NS_IMPL_NSGETMODULE(satchel, components)
+static const mozilla::Module::ContractIDEntry kSatchelContracts[] = {
+  { "@mozilla.org/satchel/form-fill-controller;1", &kNS_FORMFILLCONTROLLER_CID },
+  { NS_FORMHISTORYAUTOCOMPLETE_CONTRACTID, &kNS_FORMFILLCONTROLLER_CID },
+  { NULL }
+};
+
+static const mozilla::Module kSatchelModule = {
+  mozilla::Module::kVersion,
+  kSatchelCIDs,
+  kSatchelContracts
+};
+
+NSMODULE_DEFN(satchel) = &kSatchelModule;
 
