@@ -49,6 +49,11 @@
 #include "nsIServiceManager.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch2.h"
+#include "BasicLayers.h"
+#include "LayerManagerOGL.h"
+#include "nsIXULRuntime.h"
+#include "nsIGfxInfo.h"
+#include "npapi.h"
 
 #ifdef DEBUG
 #include "nsIObserver.h"
@@ -62,6 +67,8 @@ static PRBool debug_InSecureKeyboardInputMode = PR_FALSE;
 static PRInt32 gNumWidgets;
 #endif
 
+using namespace mozilla::layers;
+
 nsIContent* nsBaseWidget::mLastRollup = nsnull;
 
 // nsBaseWidget
@@ -91,19 +98,23 @@ nsAutoRollup::~nsAutoRollup()
 
 nsBaseWidget::nsBaseWidget()
 : mClientData(nsnull)
+, mViewWrapperPtr(nsnull)
 , mEventCallback(nsnull)
+, mViewCallback(nsnull)
 , mContext(nsnull)
 , mToolkit(nsnull)
-, mEventListener(nsnull)
 , mCursor(eCursor_standard)
 , mWindowType(eWindowType_child)
 , mBorderStyle(eBorderStyle_none)
 , mOnDestroyCalled(PR_FALSE)
+, mUseAcceleratedRendering(PR_FALSE)
+, mTemporarilyUseBasicLayerManager(PR_FALSE)
 , mBounds(0,0,0,0)
 , mOriginalBounds(nsnull)
 , mClipRectCount(0)
 , mZIndex(0)
 , mSizeMode(nsSizeMode_Normal)
+, mPopupLevel(ePopupLevelTop)
 {
 #ifdef NOISY_WIDGET_LEAKS
   gNumWidgets++;
@@ -123,6 +134,15 @@ nsBaseWidget::nsBaseWidget()
 //-------------------------------------------------------------------------
 nsBaseWidget::~nsBaseWidget()
 {
+  if (mLayerManager &&
+      mLayerManager->GetBackendType() == LayerManager::LAYERS_BASIC) {
+    static_cast<BasicLayerManager*>(mLayerManager.get())->ClearRetainerWidget();
+  }
+
+  if (mLayerManager) {
+    mLayerManager->Destroy();
+  }
+
 #ifdef NOISY_WIDGET_LEAKS
   gNumWidgets--;
   printf("WIDGETS- = %d\n", gNumWidgets);
@@ -203,6 +223,7 @@ void nsBaseWidget::BaseCreate(nsIWidget *aParent,
   if (nsnull != aInitData) {
     mWindowType = aInitData->mWindowType;
     mBorderStyle = aInitData->mBorderStyle;
+    mPopupLevel = aInitData->mPopupLevel;
   }
 
   if (aParent) {
@@ -215,11 +236,6 @@ NS_IMETHODIMP nsBaseWidget::CaptureMouse(PRBool aCapture)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsBaseWidget::Validate()
-{
-  return NS_OK;
-}
-
 //-------------------------------------------------------------------------
 //
 // Accessor functions to get/set the client data
@@ -238,6 +254,90 @@ NS_IMETHODIMP nsBaseWidget::SetClientData(void* aClientData)
   return NS_OK;
 }
 
+already_AddRefed<nsIWidget>
+nsBaseWidget::CreateChild(const nsIntRect  &aRect,
+                          EVENT_CALLBACK   aHandleEventFunction,
+                          nsIDeviceContext *aContext,
+                          nsIAppShell      *aAppShell,
+                          nsIToolkit       *aToolkit,
+                          nsWidgetInitData *aInitData,
+                          PRBool           aForceUseIWidgetParent)
+{
+  nsIWidget* parent = this;
+  nsNativeWidget nativeParent = nsnull;
+
+  if (!aForceUseIWidgetParent) {
+    // Use only either parent or nativeParent, not both, to match
+    // existing code.  Eventually Create() should be divested of its
+    // nativeWidget parameter.
+    nativeParent = parent ? parent->GetNativeData(NS_NATIVE_WIDGET) : nsnull;
+    parent = nativeParent ? nsnull : parent;
+    NS_ABORT_IF_FALSE(!parent || !nativeParent, "messed up logic");
+  }
+
+  nsCOMPtr<nsIWidget> widget;
+  if (aInitData && aInitData->mWindowType == eWindowType_popup) {
+    widget = AllocateChildPopupWidget();
+  } else {
+    static NS_DEFINE_IID(kCChildCID, NS_CHILD_CID);
+    widget = do_CreateInstance(kCChildCID);
+  }
+
+  if (widget &&
+      NS_SUCCEEDED(widget->Create(parent, nativeParent, aRect,
+                                  aHandleEventFunction,
+                                  aContext, aAppShell, aToolkit,
+                                  aInitData))) {
+    return widget.forget();
+  }
+
+  return nsnull;
+}
+
+// Attach a view to our widget which we'll send events to. 
+NS_IMETHODIMP
+nsBaseWidget::AttachViewToTopLevel(EVENT_CALLBACK aViewEventFunction,
+                                   nsIDeviceContext *aContext)
+{
+  NS_ASSERTION((mWindowType == eWindowType_toplevel ||
+                mWindowType == eWindowType_dialog ||
+                mWindowType == eWindowType_invisible ||
+                mWindowType == eWindowType_child),
+               "Can't attach to window of that type");
+
+  mViewCallback = aViewEventFunction;
+
+  if (aContext) {
+    if (mContext) {
+      NS_IF_RELEASE(mContext);
+    }
+    mContext = aContext;
+    NS_ADDREF(mContext);
+  }
+
+  return NS_OK;
+}
+
+ViewWrapper* nsBaseWidget::GetAttachedViewPtr()
+ {
+   return mViewWrapperPtr;
+ }
+ 
+NS_IMETHODIMP nsBaseWidget::SetAttachedViewPtr(ViewWrapper* aViewWrapper)
+ {
+   mViewWrapperPtr = aViewWrapper;
+   return NS_OK;
+ }
+
+NS_METHOD nsBaseWidget::ResizeClient(PRInt32 aX,
+                                     PRInt32 aY,
+                                     PRInt32 aWidth,
+                                     PRInt32 aHeight,
+                                     PRBool aRepaint)
+{
+  return Resize(aX, aY, aWidth, aHeight, aRepaint);
+}
+
 //-------------------------------------------------------------------------
 //
 // Close this nsBaseWidget
@@ -252,8 +352,6 @@ NS_METHOD nsBaseWidget::Destroy()
   if (parent) {
     parent->RemoveChild(this);
   }
-  // disconnect listeners.
-  NS_IF_RELEASE(mEventListener);
 
   return NS_OK;
 }
@@ -305,6 +403,16 @@ nsIWidget* nsBaseWidget::GetSheetWindowParent(void)
   return nsnull;
 }
 
+float nsBaseWidget::GetDPI()
+{
+  return 96.0f;
+}
+
+double nsBaseWidget::GetDefaultScale()
+{
+  return 1.0;
+}
+
 //-------------------------------------------------------------------------
 //
 // Add a child to the list of children
@@ -437,9 +545,8 @@ NS_IMETHODIMP nsBaseWidget::PlaceBehind(nsTopLevelWidgetZPlacement aPlacement,
 // merely stores the state.
 //
 //-------------------------------------------------------------------------
-NS_IMETHODIMP nsBaseWidget::SetSizeMode(PRInt32 aMode) {
-
-
+NS_IMETHODIMP nsBaseWidget::SetSizeMode(PRInt32 aMode)
+{
   if (aMode == nsSizeMode_Normal ||
       aMode == nsSizeMode_Minimized ||
       aMode == nsSizeMode_Maximized ||
@@ -456,8 +563,8 @@ NS_IMETHODIMP nsBaseWidget::SetSizeMode(PRInt32 aMode) {
 // Get the size mode (minimized, maximized, that sort of thing...)
 //
 //-------------------------------------------------------------------------
-NS_IMETHODIMP nsBaseWidget::GetSizeMode(PRInt32* aMode) {
-
+NS_IMETHODIMP nsBaseWidget::GetSizeMode(PRInt32* aMode)
+{
   *aMode = mSizeMode;
   return NS_OK;
 }
@@ -637,40 +744,173 @@ NS_IMETHODIMP nsBaseWidget::MakeFullScreen(PRBool aFullScreen)
   return NS_OK;
 }
 
-//-------------------------------------------------------------------------
-//
-// Create a rendering context from this nsBaseWidget
-//
-//-------------------------------------------------------------------------
-nsIRenderingContext* nsBaseWidget::GetRenderingContext()
+nsBaseWidget::AutoLayerManagerSetup::AutoLayerManagerSetup(
+    nsBaseWidget* aWidget, gfxContext* aTarget,
+    BasicLayerManager::BufferMode aDoubleBuffering)
+  : mWidget(aWidget)
 {
-  nsresult                      rv;
-  nsCOMPtr<nsIRenderingContext> renderingCtx;
-
-  if (mOnDestroyCalled)
-    return nsnull;
-
-  rv = mContext->CreateRenderingContextInstance(*getter_AddRefs(renderingCtx));
-  if (NS_SUCCEEDED(rv)) {
-    gfxASurface* surface = GetThebesSurface();
-    NS_ENSURE_TRUE(surface, nsnull);
-    rv = renderingCtx->Init(mContext, surface);
-    if (NS_SUCCEEDED(rv)) {
-      nsIRenderingContext *ret = renderingCtx;
-      /* Increment object refcount that the |ret| object is still a valid one
-       * after we leave this function... */
-      NS_ADDREF(ret);
-      return ret;
-    }
-    else {
-      NS_WARNING("GetRenderingContext: nsIRenderingContext::Init() failed.");
-    }  
+  BasicLayerManager* manager =
+    static_cast<BasicLayerManager*>(mWidget->GetLayerManager(nsnull));
+  if (manager) {
+    NS_ASSERTION(manager->GetBackendType() == LayerManager::LAYERS_BASIC,
+      "AutoLayerManagerSetup instantiated for non-basic layer backend!");
+    manager->SetDefaultTarget(aTarget, aDoubleBuffering);
   }
-  else {
-    NS_WARNING("GetRenderingContext: Cannot create RenderingContext.");
-  }  
-  
-  return nsnull;
+}
+
+nsBaseWidget::AutoLayerManagerSetup::~AutoLayerManagerSetup()
+{
+  BasicLayerManager* manager =
+    static_cast<BasicLayerManager*>(mWidget->GetLayerManager(nsnull));
+  if (manager) {
+    NS_ASSERTION(manager->GetBackendType() == LayerManager::LAYERS_BASIC,
+      "AutoLayerManagerSetup instantiated for non-basic layer backend!");
+    manager->SetDefaultTarget(nsnull, BasicLayerManager::BUFFER_NONE);
+  }
+}
+
+nsBaseWidget::AutoUseBasicLayerManager::AutoUseBasicLayerManager(nsBaseWidget* aWidget)
+  : mWidget(aWidget)
+{
+  mWidget->mTemporarilyUseBasicLayerManager = PR_TRUE;
+}
+
+nsBaseWidget::AutoUseBasicLayerManager::~AutoUseBasicLayerManager()
+{
+  mWidget->mTemporarilyUseBasicLayerManager = PR_FALSE;
+}
+
+PRBool
+nsBaseWidget::GetShouldAccelerate()
+{
+  nsCOMPtr<nsIPrefBranch2> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
+
+  PRBool disableAcceleration = PR_FALSE;
+  PRBool forceAcceleration = PR_FALSE;
+#if defined(XP_WIN) || defined(ANDROID) || (MOZ_PLATFORM_MAEMO > 5)
+  PRBool accelerateByDefault = PR_TRUE;
+#elif defined(XP_MACOSX)
+/* quickdraw plugins don't work with OpenGL so we need to avoid OpenGL when we want to support
+ * them. e.g. 10.5 */
+# if defined(NP_NO_QUICKDRAW)
+  PRBool accelerateByDefault = PR_TRUE;
+
+  // 10.6.2 and lower have a bug involving textures and pixel buffer objects
+  // that caused bug 629016, so we don't allow OpenGL-accelerated layers on
+  // those versions of the OS.
+  // This will still let full-screen video be accelerated on OpenGL, because
+  // that XUL widget opts in to acceleration, but that's probably OK.
+  SInt32 major, minor, bugfix;
+  OSErr err1 = ::Gestalt(gestaltSystemVersionMajor, &major);
+  OSErr err2 = ::Gestalt(gestaltSystemVersionMinor, &minor);
+  OSErr err3 = ::Gestalt(gestaltSystemVersionBugFix, &bugfix);
+  if (err1 == noErr && err2 == noErr && err3 == noErr) {
+    if (major == 10 && minor == 6) {
+      if (bugfix <= 2) {
+        accelerateByDefault = PR_FALSE;
+      }
+    }
+  }
+
+# else
+  PRBool accelerateByDefault = PR_FALSE;
+# endif
+
+#else
+  PRBool accelerateByDefault = PR_FALSE;
+#endif
+
+  if (prefs) {
+    // we should use AddBoolPrefVarCache
+    prefs->GetBoolPref("layers.acceleration.disabled",
+                       &disableAcceleration);
+
+    prefs->GetBoolPref("layers.acceleration.force-enabled",
+                       &forceAcceleration);
+
+  }
+
+  const char *acceleratedEnv = PR_GetEnv("MOZ_ACCELERATED");
+  accelerateByDefault = accelerateByDefault || 
+                        (acceleratedEnv && (*acceleratedEnv != '0'));
+
+  nsCOMPtr<nsIXULRuntime> xr = do_GetService("@mozilla.org/xre/runtime;1");
+  PRBool safeMode = PR_FALSE;
+  if (xr)
+    xr->GetInSafeMode(&safeMode);
+
+  if (disableAcceleration || safeMode)
+    return PR_FALSE;
+
+  if (forceAcceleration)
+    return PR_TRUE;
+
+  nsCOMPtr<nsIGfxInfo> gfxInfo = do_GetService("@mozilla.org/gfx/info;1");
+  if (gfxInfo) {
+    PRInt32 status;
+    if (NS_SUCCEEDED(gfxInfo->GetFeatureStatus(nsIGfxInfo::FEATURE_OPENGL_LAYERS, &status))) {
+      if (status != nsIGfxInfo::FEATURE_NO_INFO) {
+        NS_WARNING("OpenGL-accelerated layers are not supported on this system.");
+        return PR_FALSE;
+      }
+    }
+  }
+
+  if (accelerateByDefault)
+    return PR_TRUE;
+
+  /* use the window acceleration flag */
+  return mUseAcceleratedRendering;
+}
+
+LayerManager* nsBaseWidget::GetLayerManager(bool* aAllowRetaining)
+{
+  return GetLayerManager(LAYER_MANAGER_CURRENT, aAllowRetaining);
+}
+
+LayerManager* nsBaseWidget::GetLayerManager(LayerManagerPersistence,
+                                            bool* aAllowRetaining)
+{
+  if (!mLayerManager) {
+
+    mUseAcceleratedRendering = GetShouldAccelerate();
+
+    if (mUseAcceleratedRendering) {
+      nsRefPtr<LayerManagerOGL> layerManager =
+        new mozilla::layers::LayerManagerOGL(this);
+      /**
+       * XXX - On several OSes initialization is expected to fail for now.
+       * If we'd get a none-basic layer manager they'd crash. This is ok though
+       * since on those platforms it will fail. Anyone implementing new
+       * platforms on LayerManagerOGL should ensure their widget is able to
+       * deal with it though!
+       */
+      if (layerManager->Initialize()) {
+        mLayerManager = layerManager;
+      }
+    }
+    if (!mLayerManager) {
+      mBasicLayerManager = mLayerManager = CreateBasicLayerManager();
+    }
+  }
+  if (mTemporarilyUseBasicLayerManager && !mBasicLayerManager) {
+    mBasicLayerManager = CreateBasicLayerManager();
+  }
+  LayerManager* usedLayerManager = mTemporarilyUseBasicLayerManager ?
+                                     mBasicLayerManager : mLayerManager;
+  if (aAllowRetaining) {
+    *aAllowRetaining = (usedLayerManager == mLayerManager);
+  }
+  return usedLayerManager;
+}
+
+BasicLayerManager* nsBaseWidget::CreateBasicLayerManager()
+{
+#if !defined(MOZ_IPC)
+      return new BasicLayerManager(this);
+#else
+      return new BasicShadowLayerManager(this);
+#endif
 }
 
 //-------------------------------------------------------------------------
@@ -724,25 +964,11 @@ NS_METHOD nsBaseWidget::SetWindowClass(const nsAString& xulWinType)
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-NS_METHOD nsBaseWidget::SetBorderStyle(nsBorderStyle aBorderStyle)
-{
-  mBorderStyle = aBorderStyle;
-  return NS_OK;
-}
-
-
-/**
-* Sets the event listener for a widget
-*
-**/
-NS_METHOD nsBaseWidget::AddEventListener(nsIEventListener * aListener)
-{
-  NS_PRECONDITION(mEventListener == nsnull, "Null event listener");
-  NS_IF_RELEASE(mEventListener);
-  NS_ADDREF(aListener);
-  mEventListener = aListener;
-  return NS_OK;
-}
+//-------------------------------------------------------------------------
+//
+// Bounds
+//
+//-------------------------------------------------------------------------
 
 /**
 * If the implementation of nsWindow supports borders this method MUST be overridden
@@ -773,18 +999,29 @@ NS_METHOD nsBaseWidget::GetScreenBounds(nsIntRect &aRect)
   return GetBounds(aRect);
 }
 
-/**
-* 
-*
-**/
+nsIntPoint nsBaseWidget::GetClientOffset()
+{
+  return nsIntPoint(0, 0);
+}
+
 NS_METHOD nsBaseWidget::SetBounds(const nsIntRect &aRect)
 {
   mBounds = aRect;
 
   return NS_OK;
 }
- 
 
+NS_IMETHODIMP
+nsBaseWidget::GetNonClientMargins(nsIntMargin &margins)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+ 
+NS_IMETHODIMP
+nsBaseWidget::SetNonClientMargins(nsIntMargin &margins)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
 
 NS_METHOD nsBaseWidget::EnableDragDrop(PRBool aEnable)
 {
@@ -845,6 +1082,36 @@ nsBaseWidget::ShowsResizeIndicator(nsIntRect* aResizerRect)
   return PR_FALSE;
 }
 
+NS_IMETHODIMP
+nsBaseWidget::SetAcceleratedRendering(PRBool aEnabled)
+{
+  if (mUseAcceleratedRendering == aEnabled) {
+    return NS_OK;
+  }
+  mUseAcceleratedRendering = aEnabled;
+  if (mLayerManager) {
+    mLayerManager->Destroy();
+  }
+  mLayerManager = NULL;
+  return NS_OK;
+}
+
+PRBool
+nsBaseWidget::GetAcceleratedRendering()
+{
+  return mUseAcceleratedRendering;
+}
+
+NS_METHOD nsBaseWidget::RegisterTouchWindow()
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_METHOD nsBaseWidget::UnregisterTouchWindow()
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 NS_IMETHODIMP
 nsBaseWidget::OverrideSystemMouseScrollSpeed(PRInt32 aOriginalDelta,
                                              PRBool aIsHorizontal,
@@ -961,6 +1228,32 @@ nsBaseWidget::BeginResizeDrag(nsGUIEvent* aEvent, PRInt32 aHorizontal, PRInt32 a
 {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
+
+NS_IMETHODIMP
+nsBaseWidget::BeginMoveDrag(nsMouseEvent* aEvent)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+// For backwards compatibility only
+NS_IMETHODIMP
+nsBaseWidget::SetIMEEnabled(PRUint32 aState)
+{
+  IMEContext context;
+  context.mStatus = aState;
+  return SetInputMode(context);
+}
+ 
+NS_IMETHODIMP
+nsBaseWidget::GetIMEEnabled(PRUint32* aState)
+{
+  IMEContext context;
+  nsresult rv = GetInputMode(context);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  *aState = context.mStatus;
+  return NS_OK;
+}
  
 #ifdef DEBUG
 //////////////////////////////////////////////////////////////
@@ -986,7 +1279,6 @@ case _value: eventName.AssignWithConversion(_name) ; break
   switch(aGuiEvent->message)
   {
     _ASSIGN_eventName(NS_BLUR_CONTENT,"NS_BLUR_CONTENT");
-    _ASSIGN_eventName(NS_CONTROL_CHANGE,"NS_CONTROL_CHANGE");
     _ASSIGN_eventName(NS_CREATE,"NS_CREATE");
     _ASSIGN_eventName(NS_DESTROY,"NS_DESTROY");
     _ASSIGN_eventName(NS_DRAGDROP_GESTURE,"NS_DND_GESTURE");
@@ -1005,7 +1297,6 @@ case _value: eventName.AssignWithConversion(_name) ; break
     _ASSIGN_eventName(NS_KEY_DOWN,"NS_KEY_DOWN");
     _ASSIGN_eventName(NS_KEY_PRESS,"NS_KEY_PRESS");
     _ASSIGN_eventName(NS_KEY_UP,"NS_KEY_UP");
-    _ASSIGN_eventName(NS_MENU_SELECTED,"NS_MENU_SELECTED");
     _ASSIGN_eventName(NS_MOUSE_ENTER,"NS_MOUSE_ENTER");
     _ASSIGN_eventName(NS_MOUSE_EXIT,"NS_MOUSE_EXIT");
     _ASSIGN_eventName(NS_MOUSE_BUTTON_DOWN,"NS_MOUSE_BUTTON_DOWN");
@@ -1015,8 +1306,12 @@ case _value: eventName.AssignWithConversion(_name) ; break
     _ASSIGN_eventName(NS_MOUSE_MOVE,"NS_MOUSE_MOVE");
     _ASSIGN_eventName(NS_MOVE,"NS_MOVE");
     _ASSIGN_eventName(NS_LOAD,"NS_LOAD");
+    _ASSIGN_eventName(NS_POPSTATE,"NS_POPSTATE");
+    _ASSIGN_eventName(NS_BEFORE_SCRIPT_EXECUTE,"NS_BEFORE_SCRIPT_EXECUTE");
+    _ASSIGN_eventName(NS_AFTER_SCRIPT_EXECUTE,"NS_AFTER_SCRIPT_EXECUTE");
     _ASSIGN_eventName(NS_PAGE_UNLOAD,"NS_PAGE_UNLOAD");
     _ASSIGN_eventName(NS_HASHCHANGE,"NS_HASHCHANGE");
+    _ASSIGN_eventName(NS_READYSTATECHANGE,"NS_READYSTATECHANGE");
     _ASSIGN_eventName(NS_PAINT,"NS_PAINT");
     _ASSIGN_eventName(NS_XUL_BROADCAST, "NS_XUL_BROADCAST");
     _ASSIGN_eventName(NS_XUL_COMMAND_UPDATE, "NS_XUL_COMMAND_UPDATE");
@@ -1240,26 +1535,15 @@ nsBaseWidget::debug_DumpPaintEvent(FILE *                aFileOut,
   if (!debug_GetCachedBoolPref("nglayout.debug.paint_dumping"))
     return;
   
+  nsIntRect rect = aPaintEvent->region.GetBounds();
   fprintf(aFileOut,
-          "%4d PAINT      widget=%p name=%-12s id=%-8p rect=", 
+          "%4d PAINT      widget=%p name=%-12s id=%-8p bounds-rect=%3d,%-3d %3d,%-3d", 
           _GetPrintCount(),
           (void *) aWidget,
           aWidgetName.get(),
-          (void *) aWindowID);
-  
-  if (aPaintEvent->rect) 
-  {
-    fprintf(aFileOut,
-            "%3d,%-3d %3d,%-3d",
-            aPaintEvent->rect->x, 
-            aPaintEvent->rect->y,
-            aPaintEvent->rect->width, 
-            aPaintEvent->rect->height);
-  }
-  else
-  {
-    fprintf(aFileOut,"none");
-  }
+          (void *) aWindowID,
+          rect.x, rect.y, rect.width, rect.height
+    );
   
   fprintf(aFileOut,"\n");
 }
